<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <title>Conjecture - Components</title>
  <meta http-equiv="content-type" content="text/html; charset=ISO-8859-15">
  <!-- link rel="icon" href="conjecture.ico" type="image/x-icon" -->
  <!-- link rel="shortcut icon" href="conjecture.ico" type="image/x-icon" -->
  <link rel="stylesheet" href="style.css" type="text/css" title="plain">
</head>
<body>
<table class="nopad" align="left" border="0" cellpadding="0"
 cellspacing="0" width="100%">
  <tbody>
    <tr>
      <td class="nopad">
<?php
 $mainvalue='Documentation';
 $subvalue='Components';
 include_once("header.php");
?>
      </td>
    </tr>
    <tr>
      <td>

      <table class="nopad" align="left" border="0" cellpadding="0" cellspacing="0" width="100%">
        <tbody>
          <tr valign="top">
            <td class="content">

            <div id="overview">

<div id="overview">
  <h1> An Overview of Components </h1>
  <blockquote>

    <p>The Conjecture framework provides a collection of kernel classes
    (including the \ref kernel "Element" hierarchy discussed elsewhere).
    Classes like Page, Line, and Glyph are of use to any OCR
    implementation. Furthermore, the Page class provides fundamental
    methods like 'segment' (responsible for identifying Glyphs within the
    Image associated with the Page), 'identify' (responsible for
    identifying unicode characters for each Glyph), and 'format'
    (responsible properly formatting the identified characters into words,
    lines, paragraphs, columns, pages, etc.)

    <p>In order to explain both the overall Conjecture architecture, and the
    rationale behind the architecture, we will first look at some initial
    solutions to the problem of customization. The Conjecture framework is
    supposed to make it easy for individuals to contribute anything from
    tiny within-method improvements to single-method reimplementations to
    entirely new algorithms to entirely new fully-functional OCRs.

    <p>Suppose we limit ourselves to just the Element hierarchy for now, and
    consider how an individual can extend the framework. Since the Page
    class defines issue-solving methods like 'segment()', 'identify()',
    and 'format()', one natural means of customization is to introduce a
    subclass of Page and redefine the methods one wants to experiment
    with. For example, an individual could create a MyPage class, and
    could redefine 'segment() in order to write a better glyph segmenting
    algorithm, or redefine 'identify()' in order to write a better glyph
    identification algorithm. This strategy is both straight-forward and
    intuitive. However, it also suffers from a serious drawback.
    Specifically, there are many possible implementations of the 'segment'
    algorithm, many possible implementations of the 'identify' algorithm,
    and in general many different implementations of all the various
    issues related to OCR. Suppose there are K variants for 'segment()', L
    variants of 'identify' and M variants of 'format()'. Using the
    strategy of subclassing Page and redefining relevant methods to
    provide extension, we would need to create K * L * M 'Page' subclasses
    in order to represent all combinations of the various implementations.
    Depending on the values of K, L and M, this number can quickly become
    prohibitive, and that assumes there will only be 3 issue-solving
    methods defined on Page (which isn't true at all - there will probably
    be 20 or more such methods, each with many possible implementations).

    <p>Having explained why subclassing the Element hierarchy to support
    extension can be problematic, we can now explain what the Conjecture
    framework does to address the problem. As is usually the case, adding
    a level of abstraction makes everything sunny and happy. Although this
    extra level of abstraction makes the overall architecuture more
    complex, and thus somewhat more difficult to grasp at first, the
    flexibility it provides in allowing incremental improvements is well
    worth the increase in complexity.

    <p>As an example, Conjecture has formalized the concept of 'segmentation'
    (finding Glyphs within an Image) into a SegmentComponent class
    hierarchy. The abstract SegmentComponent class defines an interface,
    and subclasses provide an implementation of that interface.
    Similarily, the concept of 'identification' (establishing unicode
    values for each Glyph) has been formalized into the IdentifyComponent,
    and so on. As new issues are identified, new component hierarchies
    will be added to the Conjecture framework.

    <p>So far, Conjecture has identified three fundamental issues: Segment,
    Identify and Format. However, Conjecture will be providing
    decompositions of each of these components into sub-components. For
    example, dust removal, page orientation, and font-detection are all
    possible sub-issues of segmentation that could be formalized into
    Component hierarchies.

    <p>The fact that components can be implemented in terms of other
    components highlights some important issues. First, it implies that
    there are some constraints between components, and the order in which
    certain components are performed may (or may not) be important,
    depending on the particular component(s) in question. Furthermore, we
    have already demonstrated the decomposition of components by
    implicitly assuming that every OCR will perform segmentation,
    identification and formatting. Although this is probably true, in the
    interests of completely generality, Conjecture has also formalized the
    entire OCR process into a ProcessModule. More on this later.

    <p>The Component-related class diagram of Conjecture currently looks like
    this:


      <div id="uml-components-hier">
         <?php include_once("uml/components-hier.map"); ?>
         <center><img src="uml/components-hier.jpg" usemap="#components-hier"></center>
      </div>

    <p>Notice that the four issues that Conjecture has currently formalized
    exist as subclass hierarchies of an abstract Component class. The
    ProcessComponent class defines an interface (collection of methods)
    that together are responsible for the entire optical character
    recognition process. Currently, the interface for all classes consists
    of a single 'execute(Page* page)' method, although more methods may be
    added to individual Component hierarchies as the framework evolves.

    <p>Subclasses of the Component hierarchies provide alternative
    implementations. For example, the StandardProcess subclass of
    ProcessComponent provides what will probably be the most common
    implementation of the Process component - perform segementation, then
    identification, then formatting. This default implementation knows
    that the concepts of segmentation, identification and formatting have
    been formalized into Components of their own, and thus, when it wants
    to perform segmentation, asks the Conjecture environment for an object
    that implements the SegmentComponent interface. Once it has obtained
    this object, it can invoke the 'execute(Page*page)' method on it
    (because it knows that this method is part of the SegmentComponent
    interface). The StandardProcess class needs to know absolutely nothing
    about the actual subclass of SegmentComponent being used in order to
    provide segmentation. This means that the StandardProcess class can
    work with any SegmentComponent (and any IdentifyComponent and
    FormatComponent), making the framework highly modular and flexible.

    <p>The OcradProcess subclass of ProcessComponent does something different
    than StandardProcess. Instead of decomposing the problem into
    segmentation, identification and formatting, this class instead
    invokes the "ocrad" executable as a subprocess. Whether 'ocrad'
    decomposes the problem into segmentation, identification and
    formatting is irrelevant from the perspective of the OcradProcess
    class. All it knows is that it is responsible for taking an input
    image and producing output text. It knows that the 'ocrad' executable
    can do that, and thus, by invoking 'ocrad', it satisfies its mandate.

    <p>The OcradProcess is interesting because it demonstrates how Conjecture
    can be used to interact with arbitrary third-party OCRs (including
    even commercial ones, as long as they have a command-line interface).
    Note also that the current implementation of OcradProcess will be
    changing soon. Although delegation to an executable is sometimes
    useful, Conjecture is about identifying and extending existing
    algorithms for solving OCR problems, and as such it is much better to
    link the Ocrad source code into Conjecture so that programmers have
    access to the internals and can incrementally improve upon or borrow
    from its implementation in the creation of new algorithms.

      <div id="uml-components-pub">
         <?php include_once("uml/components-pub.map"); ?>
         <center><img src="uml/components-pub.jpg" usemap="#components-pub"></center>
      </div>

  </blockquote>
</div>

<div id="existing">
  <h1>OCR Components (and Implementations) Currently Available in Conjecture</h1>
  <blockquote>

    <div id="process">
      <h2>The Process Component</h2>
      <blockquote>
        <ul>
          <li> <b>Purpose</b>: Convert input image to output text</li>
          <li> <b>Decomposes</b>: None (top-level component)</li>
          <li> <b>Pre</b>:
            <ul>
              <li> Env::module is initialized
              <li> Env::params is initialized 
            </ul>
          </li>
          <li> <b>Post</b>:
            <ul>
              <li> Textual output is written to params()->outfile()
            </ul>
          </li>
        </ul>

        This component represents the entire process of converting an
        input image into output text. It is almost always decomposed
        into sub-components, and in particular is usually implemented
        using the <code>StandardProcess</code> subclass, which invokes
        the <em>segment</em>, <em>identify</em> and <em>format</em>
        components in sequence.

        <p>The <code>ExternalProcess</code> implementation of the
        <code>Process</code> component can be used to create a
        module that wraps an externally invoked OCR. [wmh:
        <code>ExternalProcess</code> not yet implemented - see
        <code>OcradProcess</code>, which will be generalized].

      </blockquote>
    </div>

    <div id="segment">
      <h2>The Segment Component</h2>
      <blockquote>
        <ul>
          <li> <b>Purpose</b>: Partition a Page into Glyphs</li>
          <li> <b>Decomposes</b>: Process </li>
          <li> <b>Pre</b>:
            <ul>
              <li> All pre-conditions of <code>process</code>, plus
              <li> The <code>Env::currentPage()</code> method
                   returns a valid Page instance for the input
                   image to be processed.
            </ul>
          </li>
          <li> <b>Post</b>:
            <ul>

              <li>
               Invoking <code>Page::allGlyphs()</code> returns
              the set of all <code>Glyph</code> instances to be
              analyzed.
              </li>

            </ul>
          </li>
        </ul>

        <p>This component is not required to create
        <code>Region</code>, <code>Line</code> and/or
        <code>Word</code> page partitionings, but is not precluded
        from doing so.

        <p>It is possible that subsequent components will delete,
        split, or mark as invisible, Glyph instances identified here.
        This component just provides a preliminary set of Glyphs to
        process.
      </blockquote>
    </div>

    <div id="identify">
      <h2>The Identify Component</h2>
      <blockquote>
        <ul>
          <li> <b>Purpose</b>: Establish unicode characters for each Glyph </li>
          <li> <b>Decomposes</b>: Process</li>
          <li> <b>Pre</b>:
            <ul>
              <li> The initial set of <code>Glyph</code> instances
              to analyze is available via <code>Page::allGlyphs()</code>
            </ul>
          </li>
          <li> <b>Post</b>:
            <ul>
              <li> Every <code>Glyph</code> has its <code>unicode</code>
              field set.
            </ul>
          </li>
        </ul>

        <p> This component may modify the initial set of
        <code>Glyph</code> instances (by deleting or marking as
        invisible glyphs identified as noise, and by splitting glyphs
        identified as being two or more separate glyphs considered one
        during segmentation).

      </blockquote>
    </div>

    <div id="format">
      <h2>The Format Component</h2>
      <blockquote>
        <ul>
          <li> <b>Purpose</b>: Produce formatted text on output</li>
          <li> <b>Decomposes</b>: Process </li>
          <li> <b>Pre</b>:
            <ul>
              <li> Every <code>Glyph</code> has its <code>unicode</code>
              field set.
            </ul>
          </li>
          <li> <b>Post</b>:
            <ul>
              <li> The command-line-specified output file contains
              text representing the image.
            </ul>
          </li>
        </ul>
          
        <p>In order to properly format the output text, the
        <code>Page</code> must, before output is generated, be more
        richly divided so that it contains a deeper containment
        structure than just a collection of <code>Glyph</code>
        images in the <code>Page</code>. In particular, each
        <code>Glyph</code> is usually wrapped into a
        <code>Word</code>, which is wrapped into a
        <code>Line</code>, which may be wrapped into a
        <code>Region</code> ( (if image represents multi-column
        text, for example).

        <p> It is possible that previous components (segment or
        identify) have already provided some or all of this deep
        decomposition. It is also possible that previous components
        have NOT done so. In order to ensure component independence,
        the implementations of <code>FormatComponent</code> should
        never assume either way, and should support arbitrary
        pre-existing decompositions.

        <p>Note that the <code>Element::writeText</code> method
        makes the actual output of text trivial, so the real work
        assigned to <code>FormatComponent</code> is the
        establishment of the proper Element containment hierarchy.

      </blockquote>
    </div>
  </blockquote>
</div>

<div id="implementations">
  <h1>Creating a new Implementation of an Existing Component</h1>
  <blockquote>

    <p> Remember that Conjecture formalizes issues important to OCR
    into <em>Components</em>. However, the classes representing a
    Component are just interfaces. It is the subclasses of these
    classes that implement the interfaces. These subclasses are
    refered to as <em>Component Implementation</em>, abbreviated
    simple as <em>Implementations</em>. The goal is to design
    Conjecture such that <em>any</em> subclass of a Component
    interface can be used to implement its interface - complete
    independence from all other components. In practice, this is
    sometimes complicated by poorly designed third-party code, but
    complete separate and independence is the goal.

    <p>As an example, if a new implementation of the
    <code>SegmentComponent</code> component is being defined, then the
    new class is responsible for satisfying the post-conditions of
    <code>SegmentComponent</code> (see the <A
    HREF="#components">existing components</A> section for details on
    post-conditions). The interface of existing components concists
    simply of a <code>execute(Element*)</code> method, and
    implementation of the interface consists in implementing this
    single method. Naturally, the class can introduce as many
    additional methods (and service classes) as necessary in order to
    implement this <code>execute</code> method.

    <p>Conjecture provides a means of automatically generating a
    properly structure class skeleton for new components as part of
    the creation/verification of Modules. Please see the documentation
    on <A HREF="modules.php#creating">Creating a New OCR Module</A>
    for more details on creating component implementations.

  </blockquote>
<div>

<div id="creating">
  <h1>Creating new OCR Components</h1>
  <blockquote>

    <p> There are many issues related to OCR that have not yet been
    formalized into components. More specifically, the exsting
    Segment, Identify and Format components can each be decomposed
    into a variety of sub-components. However, the exact set of issues
    to be formalized into components is yet to be established, and is
    not as obvious as the decomposition of Process into
    Segment/Identify/Format.

    <p>For example, one possible subcomponent of Segment is
    ImageCanonicalization (responsible for thresholding, pixel
    filtering, dust removal, etc.). Alternatively, each of these could
    be sub-components of Segment themselves (ThresholdComponent,
    PixelFilterComponent, DustRemovalComponent, etc.), or even
    sub-sub-components of ImageCanonicalization. More design work is
    needed here!

  </blockquote>
<div>

            </td>

            <!-- begin side bar -->
            <td class="right" width="200">
<?php
 include_once("sidebar.php");
?>

            </td>

            <!-- end side bar -->

          </tr>
      </table>
      </td>
    </tr>
    <tr>
      <td class="nopad">
<?php
 include_once("footer.php");
?>
      </td>
    </tr>
  </tbody>
</table>


</body>
</html>
