<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <title>Conjecture - Documentation - Harness</title>
  <meta http-equiv="content-type" content="text/html; charset=ISO-8859-15">
  <!-- link rel="icon" href="conjecture.ico" type="image/x-icon" -->
  <!-- link rel="shortcut icon" href="conjecture.ico" type="image/x-icon" -->
  <link rel="stylesheet" href="style.css" type="text/css" title="plain">
</head>
<body>
<table class="nopad" align="left" border="0" cellpadding="0"
 cellspacing="0" width="100%">
  <tbody>
    <tr>
      <td class="nopad">
<?php 
 $mainvalue='Documentation';
 $subvalue='Harness';
 include_once("header.php");
?>
      </td>
    </tr>
    <tr>
      <td>
      
      <table class="nopad" align="left" border="0" cellpadding="0" cellspacing="0" width="100%">
        <tbody>
          <tr valign="top">
            <td class="content">



<div id="overview">
  <h1>The Testing Harness of Conjecture</h1>
  <blockquote>

    <p>Conjecture encourages users to contribute everything from tiny
    modifications in existing methods of existing classes, to full-fledged
    OCR Modules. However, modifications to the codebase can potentially
    affect more than is immediately apparent. Testing a change by using a
    single set of command-line flags, for a single OCR module, on a single
    input image, does not accurately assess the overall impact that a
    change has on the Conjecture environment as a whole.

    <p>For this reason, Conjecture provides an assessment infrastructure with
    the following features:

     <ul>

       <li> A (arbitrarily large) collection of input images

       <li> A corresponding collection of text files representing what an
            OCR implementation should produce, if it had 100% accuracy.

       <li> The ability to test individual OCRs or groups of OCRs (or all OCRs),
            against individual images or groups of images (or all images).

       <li> Every OCR Module has an associated set of <em>configurable parameters</em>,
            values that somehow influence performance and can be
            configured by the user. A particular combination of values for
            all configurable parameters represents a single <em>module
            variant</em>. The assessment infrastructure allows one to test
            a specific variant or a group of variants. Note that the
            complete set of variants possible is effectively infinite
            because some parameters can take on floating point values).
            For this reason, Conjecture provides a way of enumerating and
            naming variants that produce interesting results.

       <li> Can establish the accuracy of each variant of each OCR.

       <li> Can report the affect that a change in the code base has
            on the entire execution environment (increased or decreased
            accuracy, increased or decreased time-to-compute, etc.)

    </ul>

    <div id="directory">
      <h2> The <code>$CONJECTURE/harness</code> directory structure </h2>
      <blockquote>

        <code>$CONJECTURE/harness</code> has three important components:

          <ul>

           <li> The <code>../config/Conjecture.tests</code> file, describing
           algsets and algset variants to test. [wmh: this filename will be
           changing!]

           <li> The <code>db</code> subdir

           <li> The <code>custom</code> subdir

         </ul>
      </blockquote>
    </div>

    <div id="testsfile">
      <h2> The <code>$CONJECTURE/config/Conjecture.tests</code> file </h2>
      <blockquote>
        <p> This file describes the set of OCR Modules and module variants to test.

        <div class="fragment"><pre class="fragment">
  # this is the default module, 
  # used if -M is not specified
  MODULE default
    VARIANT v1;

  MODULE gocr
    # variant 1 of the gocr module
    VARIANT v1;

  MODULE ocrad  # a comment, ignored
    VARIANT v1;
    VARIANT v2 FLAGS { -Z "-T .7" };
    VARIANT v3 FLAGS { -Z "-T .75" };
         </pre></div>

        <p> In this file, anything after a '#' (including the '#')
        is removed as a comment).  The <code>ocrgen</code> ignores
        blank lines in this file.   Except for the above, there are only
        two legal line formats within this file (any other line format
        is an error and is ignored by <code>ocrgen</code>).
     
        <ol> 

         <li> an line containing the keyword <code>MODULE</code>
            followed by &lt;module&gt; name. The &lt;module&gt;
            must be a legal identifer (start with a letter or
            underscore, and continue with letters, underscores or
            digits) <em>and</em> correspond to the name of Module
            defined in
            <code>$CONJECTURE/config/Conjecture.modules</code>.
            [<b>wmh: the .modules and .tests files will probably be merged later</b>]

         <li> a line containing the keyword <code>VARIANT</code>,
            followed by a variant name, &lt;variant&gt;, followed
            optionally by the keyword <code>FLAGS</code> an open
            brace, an arbitrary sequence of <code>conjecture</code>
            command line flags, a close brace, and a semicolon. The
            variant name must be a legal identifier.  If
            <code>FLAGS</code> is not provided, default values are
            used for all flags.  If it is provided, the open/close
            braces, flags, and semicolon must be provided.

         </ol>

         <p> Note that the value associated with <code>FLAGS</code>
         does not represent the entire set of flags sent to
         <code>conjecture</code>. In particular, the following flags
         are implicitly provided by the test harness during each
         invocation of <code>conjecture</code> generated by the
         test harness.

         <pre>
  -V 0 
  -M &lt;module&gt; 
  -i &lt;input&gt;.&lt;suffix&gt; 
  -o custom/&lt;module&gt;/&lt;variant&gt;.ocr
         </pre>

         <p>All of these flags are added <em>after</em> the flags from
         the <code>VARIANT</code> (and thus override any attempts to
         define them within <code>FLAGS</code>).

         <p><em>[wmh: In general, XML is the proper way to describe
         input formats nowadays. However, I wanted this file to be
         accessible to non-programmers, and didn't want to force them
         to learn XML. By defining a very simple format, with only two
         kinds of acceptable lines, it is hoped that
         programming-phobic individuals will be able to edit and
         experiment with Conjecture at this level.  The format is
         actually based on a xml-equivalent syntax (related to
         research completely unrelated to OCRs).]</em>
      </blockquote>
    <div>
 
    <div>
      <h2> The <code>harness/db</code> directory </h2>
      <blockquote>

         <p>This directory contains an arbitrarily large collection of input
         image files in various graphics formats. Each input image has a
         corresponding <code>,valid</code> file providing the text
         representation of the image file. The <code>.valid</code> file
         represents the desired output that should be produced by an OCR
         that has 100% accuracy.

         <p> One ramification of the above is that if two image files have the same
         prefix, with different suffixes (.jpg vs .pnm, for example), both
         images must refer to exactly the same textual content, since both
         files will be described by the single associated .valid file.
       </blockquote>
    <div>

    <div id="customdir">
      <h2> The <code>harness/custom</code> directory </h2>
      <blockquote>

        <p>The <code>custom</code> directory contains subdirectories for each
        OCR module, &lt;module&gt;. Within each of these directories, a
        subsubdirectory exists for each variant, &lt;variant&gt;.

        <p> The test harness involves executing <code>conjecture</code> on many
        input files using many command-line variants. The output from each
        run is placed into
        <code>harness/ custom/ &lt;module&gt;/ &lt;variant&gt;/ &lt;input&gt;.ocr</code>.

        <p> The variant subdirectories contain, in addition to
        <code>.ocr</code> files, <code>&lt;input&gt;.val</code> files.
        These files are similar to
        <code>harness/db/&lt;input&gt;.valid</code> files, except that
        rather than representing the perfect output, they represent
        the "expected" result for the given module and variant. These
        files are used in order to quickly determine whether a change
        in the code base has had an effect (intentional or otherwise)
        on the performance of a specific module variant.
      </blockquote>
    </div>
  </blockquote>
</div>

<div id="ocrtest">
  <h1> The <code>ocrtest</code> Script </h1>
  <blockquote>

    <p> The <code>ocrtest</code> script is the interface to the Conjecture
    test harness. It is usually invoked with one of these purposes in
    mind:
     <ul>
       <li> validity testing
       <li> assessment testing
       <li> comparison testing
       <li> individual testing
     </ul>

    <div id="validity">
      <h2> Validity Testing </h2>
      <blockquote>
        <p>When changes to the codebase have been performed, it is important to
        know the overall impact these changes have had. Testing on a single
        input file with a single combination of flags does not guarantee that
        a change doesn't introduce unintentional affects that detrimentally
        (or positively) affects some other module and/or variant.

        <p>Using the command:

        <pre>
  % ocrtest -a
        </pre>

        runs the test harness over every module and variant specified
        in <code>$CONJECTURE/ config/ Conjecture.tests</code>, for
        every image file found in <code>$CONJECTURE/harness/db</code>,
        writing results into <code>harness/ custom/ &lt;module&gt;/
        &lt;variant&gt;/ &lt;input&gt;.ocr</code>. This result is then
        compared against <code>harness/ custom/ &lt;module&gt;/
        &lt;variant&gt;/ &lt;input&gt;.val</code>. A 2D table is then
        generated, with module/variants as rows, and input files as
        columns. Within the table, if the module/variant/input
        produces an <code>.ocr</code> that matches <code>.val</code>
        exactly, the table entry will contain plus ('+') sign. If they
        do not match, this is indicated by an asterisk ('*') sign.
        Numerous other characters are used to indicate others kinds
        of errors ('@' if <code>.ocr</code> was generated, '%' if
        <code>.val</code> file is missing, '?' if the executable
        exited abnormally).  

        <p>An example of the output produces by <code>ocrtest</code>
        during verification is shown below:

        <pre>
   ============================================
   Name      4x6   5x7   5x8 ocr-a ocr-b   rod 
   --------------------------------------------
   default                                     
     v1        +     +     +     +     +     + 
   gocr                                        
     v1        +     +     +     +     +     + 
   ocrad                                       
     v1        +     +     +     +     +     + 
     v2        +     +     +     +     +     + 
     v3        +     +     +     +     +     + 
   --------------------------------------------
   Table: Conjecture Test Harness
   ============================================
        </pre>

        <p>If the table contains all '+', then the entire application
        is working exactly as expected. However, if any '*' appear (or
        any of a number of other symbols), it indicates that
        unexpected output has occurred. Whether this is a good thing
        or a bad thing depends on whether the difference represents an
        increase in accuracy or not, which is established using
        assessment testing, discussed next.

        <p><em>[wmh: document the other symbols that have been
        added!]</em>

        <p>The <code>ocrtest</code> script accepts a <code>-x</code>
        argument that specifies the specific <code>conjecture</code>
        executable to use. By default, it is <code>conjecture</code>,
        and the PATH environment variable is used to establish where
        to find it. However,
        <code>$CONJECTURENROOT/harness/Makefile</code> explicitly
        specifies a <code>-x ./ocrprog</code> flag to its invocations
        of <code>ocrtest</code>. Various targets in
        <code>$CONJECTURE/src</code> copy executables to
        <code>$CONJECTURE/harness/ocrprog</code> when appropriate.

      </blockquote>
    </div>

    <div id="assessment">
      <h2> Assessment Testing </h2>
      <blockquote>

        <p>The goal of an OCR is to produce accurate results. The assessment
        capabilities of the Conjecture test harness allow one to see how every
        module variant performs across all input files, by reporting the
        accuracy (as a percentage correct relative to the expected output).

        <p>Assessment teseting allows us to identify the best module variant for
        each input file. As well, when changes to the code base have produced
        changes in output (as indicated by validity testing discussed above),
        assessment testing allows one to see whether the changes were an
        improvement or a hindrance to overall accuracy.

        <p>Using the command:

        <pre>
  % ocrtest -A
        </pre>

        <p>does everything that validity testing does, but in addition, it
        executes the <code>ocrdiff</code> to compare
        <code>harness/custom/&lt;module&gt;/&lt;variant&gt;/&lt;input&gt;.val</code>
        against <code>db/&lt;input&gt;.valid</code>. Remember that
        <code>db/&lt;input&gt;.valid</code> represents the "goal" output that
        an ocr with 100% accuracy would produce. The <code>ocrdiff</code>
        script performs an intelligent character-by-character analysis to
        establish the accuracy of the ocr output.

        <p>The result of assessment testing is another table, like this:

        <pre>
   ============================================
   Name      4x6   5x7   5x8 ocr-a ocr-b   rod 
   --------------------------------------------
   default                                     
     v1      87    97    98    88   100    80  
   gocr                                        
     v1      87    97    98    88   100    80  
   ocrad                                       
     v1       0     0     0    49   -22    57  
     v2       0     0     0    49   -22    54  
     v3       0     0     0    49   -22    57  
   --------------------------------------------
   Table: Conjecture Test Harness
   ============================================
        </pre>

        <p>It reports the percentage accuracy for each module variant when
        applied to every input image. It also reports the validity
        information, except that anytime a '+' would have been shown in the
        validity table, a space (' ') is shown (to avoid unnecessary clutter).

        <p><em>[wmh: update this table, explain why the -22 is occuring, etc.]</em>

      </blockquote>
    </div>

    <div id="comparison">
      <h2> Comparison Testing </h2>
      <blockquote>

        <p>Comparison testing is very similar to Assessment testing:

        <pre>
   % ocrtest -A -x &lt;newexec&gt; -X &lt;oldexec&gt;
        </pre>

        <p>The only difference is that assessment results are computed for two
        different executables. Individual tables are shown for each. And then
        a "difference" table is presented showing the accuracy of the first
        minus the accurace of the second. This gives a convenient means of
        assessing at a glance the relative impact that a particular change in
        the code base has had on overall performance. Positive entries in this
        table indicate improvements in accuracy, while negative entries
        indicate a worsening of accuracy.

        <p>Naturally, comparison testing assumes that you have two different
        executables to compare. When you are experimenting with a new
        algorithm, you should always make a copy of the "baseline" executable
        (and any other incremental improvements along the way) so that you
        will have them available for comparison testing.
      </blockquote>
    </div>
  </blockquote>
</div>

            </td>

            <!-- begin side bar -->
            <td class="right" width="200">
<?php 
 include_once("sidebar.php");
?>

            </td>
            
            <!-- end side bar -->

          </tr>
      </table>
      </td>
    </tr>
    <tr>
      <td class="nopad">
<?php 
 include_once("footer.php");
?>
      </td>
    </tr>
  </tbody>
</table>


</body>
</html>
