<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <title>Conjecture - Documentation - Kernel Classes</title>
  <meta http-equiv="content-type" content="text/html; charset=ISO-8859-15">
  <!-- link rel="icon" href="conjecture.ico" type="image/x-icon" -->
  <!-- link rel="shortcut icon" href="conjecture.ico" type="image/x-icon" -->
  <link rel="stylesheet" href="style.css" type="text/css" title="plain">
</head>
<body>
<table class="nopad" align="left" border="0" cellpadding="0"
 cellspacing="0" width="100%">
  <tbody>
    <tr>
      <td class="nopad">
<?php 
 $mainvalue='Documentation';
 $subvalue='Kernel';
 include_once("header.php");
?>
      </td>
    </tr>
    <tr>
      <td>
      
      <table class="nopad" align="left" border="0" cellpadding="0" cellspacing="0" width="100%">
        <tbody>
          <tr valign="top">
            <td class="content">


<div id="kernel">
  <h1>The Kernel Classes</h1>

    <p>The Conjecture framework defines a core collection of C++
    classes that represent the fundamental objects that any OCR needs.
    The kernel classes consist primarily of <code>Image</code> and the
    <code>Element</code> hierarchy, but also includes various other
    fundamental classes.  New classes will be considered for inclusion
    in the kernel based on their general utility.

  <div id="element">
    <h2> The Element Class Hierarchy </h2>
    <blockquote>

      The <code>Element</code> hierarchy is at the absolute center of OCR
      processing. Instances of the classes in this hierarchy are used to
      describe the fundamental concepts that an OCR will operate on.

      <div id="uml-kernel-hier">
        <?php include_once("uml/kernel-hier.map"); ?>
         <center><img src="uml/kernel-hier.jpg" usemap="#kernel-hier"></center>
      </div>

     <h3> Class Summary </h3>

      <table>
        <tr>
          <th>Root</th>
          <td>
          Abstract superclass of all classes in the Conjecture
          framework.
          </td>
        </tr>

        <tr>
          <th>Env</th>
          <td>
          A class providing a public interface to other
          programs.
          </td>
        </tr>

        <tr>
          <th>Image</th>
          <td>
          A collection of pixels. In-memory representation of
          a file in a known graphics format.
          </td>
        </tr>

        <tr>
          <th>Element</th>
          <td>
          Abstract superclass of a collection of classes representing a
          part-whole decomposition of a graphic image into smaller and smaller
          semantic units.
          </td>
        </tr>

        <tr>
          <th>Page</th>
          <td>
          An Image and associated meta-data representing an entire page of
          to-be-scanned data. May be sub-divided into Regions, Lines, Words
          and/or Glyphs, although only Glyphs are crucial.
          </td>
        </tr>

        <tr>
          <th>Region</th>
          <td>
          A sub-region of a Page, usually used to represent the graphical area
          for a single column in a multi-column image. May be sub-divided into
          Lines, Words, and/or Glyphs. This class is not currently prioritized
          and may be deleted if deemed unnecessary.
          </td>
        </tr>

        <tr>
          <th>Line</th>
          <td>
          A sub-region of a Page (or Region) consisting entirely of Glyphs. My
          be sub-divided into Words that contain Glyphs, but may often contain
          only Glyphs.
          </td>
        </tr>

        <tr>
          <th>Word</th>
          <td>
          A sub-region of a Line consisting entirely of Glyphs, separated from
          other Words within the Line by more horizontal space than
          </td>
        </tr>

        <tr>
          <th>Glyph</th>
          <td>A sub-region of a Page (or Region or Line or Word)
          representing a single to-be-identified character.
          </td>
        </tr>
      </table>

      <div id="uml-kernel-pub">
        <?php include_once("uml/kernel-pub.map"); ?>
         <center><img src="uml/kernel-pub.jpg" usemap="#kernel-pub"></center>
      </div>

    </blockquote>
  </div>

  <div id="isa_hasa">
    <h2> The Distinction between the <code>is-a</code> and <code>has-a</code> Element hierarchies </h2>
    <blockquote>

      <p>Note that the Element class is an aggregation of one or more instances
      of itself. This design allows for a great deal of flexibility, and
      significant code reuse between Element and its subclasses. However,
      the design does mean that there is both an Element is-a hierarchy (the
      collection of classes shown above and how they are hierarhically
      related) and an Element has-a hierarchy (the collection of specific
      objects and how they are connected to one another). It is important to
      keep this distinction between the two hierarchies clear. To this end,
      in Conjecture documentation we will refer to the Element class hierarchy
      (for the is-a relationship) and to the Element containment hierarchy
      (for the has-a relationship).

      <p>Element defines both a 'parent' field (pointing to the Element within
      which this Element is contained) and a 'parts' field (containing all
      the "child" elements). Note that in this context, the terms 'parent'
      and 'child' do NOT refer to the class hierarchy, but instead to the
      containment (object) hierarchy. For example, if we say that a Page
      contains two Regions, each of which contains 40 Lines, each of which
      contains a variety of Words, each of which contains a variety of
      Glyphs, we are talking about the hierarchy of <em>objects</em>, not
      the hierarchy of classes. At the class level, Pages, Regions, Lines,
      Words and Glyphs are all "equal" subclasses of Element, but at the
      object level, there is an inherent asymmetry due to the semantics of
      the classes; a Page can contain Regions or Lines or Words or Glyphs,
      but a Glyph cannot contain a Word or Line or Region or Page, a Line
      cannot contain a Region or Page, etc.

      <p>One way to enforce this asymmetry would have been to NOT define a
      'parts' field in Element (containing Elements), but instead to have
      the Page class define a 'regions' field containing Region instances,
      have Region define a 'lines' field containing Line instances, have
      Line define a 'words' field containing Word instances, etc. Although
      this alternative allows for more specificity, and thus better
      compile-time type-checking, it is also very constraining because it
      forces every OCR to always divide each Page into Regions into Lines
      into Words into Glyphs. Because Conjecture is attempting to be a
      universal framework that can support any OCR implementation
      imaginable, we do not want to place unnecessary restrictions on how an
      implementation performs its duties. All that is strictly needed is to
      divide Pages into Glyphs (the subdivision into Word, Line and Region
      are not strictly necesssary).

      <p>So, instead of using the above strategy, Element defines a 'parts'
      field, which allows Pages to contain Regions, Lines, Words or Glyphs,
      allows Regions to contain Lines, Words, or Glyphs, and allows Lines to
      contain Words or Glyphs, which is more flexibile. Of course, from a
      compile-time perspective this design is problematic because
      nonsensical containments are also possible (a Glyph could contain a
      Word or Line or Region or Page, a Line could contain a Region or Page,
      etc).  However, this is easily addressed with some run-time checks
      in the 'element-adding' functionality defined on the Element class.
      The increased flexibility of this approach was deemed worth the
      reduction in compile-time typechecking accuracy.

    </blockquot>
  </div>

  <div id="elemimage">
    <h2> How the Element Hierarchy interacts with Image </h2>
    <blockquote>

      <p>The overall idea behind Elements is a part-whole decomposition of an
      input image into smaller and smaller images. The manner in which this
      decomposition is implemented can have significant time and space
      efficiency ramifications.

      <p>One approach is to have each Element subclass maintain a local
      copy of that portion of the overall image to which it applies. Fields
      'height' and 'width' would establish the pixel dimensions, and a
      'data' field could store the actual pixel information. For example,
      each Glyph could maintain a copy of that portion of the Page
      represented by the Glyph.

      <p>The problem with the above naive implementation is that it incurs more
      and more memory the more sub-divided an Element becomes. Although it
      is common for Pages to contain just a collection of Glyphs, it is also
      possible for a Page to contain Regions that contain Lines that contain
      Words that contain Glyphs (which might contain other Glyphs!). In such
      a situation, the pixel data representing an individual Glyph would be
      copied (and maintained in memory) up to 5 times (stored in a Glyph,
      stored in a Word, stored in a Line, stored in a Region, and stored in
      the Page.

      <p>The above memory impact can be avoided by taking a different approach.
      Instead of each Element maintaining a separate copy of its portion of
      an image, we note that each input image corresponds to a Page, and
      thus a Page represents the largest image. In the containment
      hierarchy, all other subclasses of Element eventually "belong" to a
      Page. For this reason, if Page stores an Image instance, then every
      Element subclass has access to that "big picture" image by following
      its 'parent' field up until a Page is reached (which is guaranteed by
      the Conjecture framework to always occur). Each Element can be thought
      of as representing a specific rectangular region within that "big
      picture" image associated with the Page.

      <p>By having each Element store the top-left and bottom-right coordinates
      (relative to the "big-picture" image), we can avoid requiring each
      element to maintain individual copies. It is for this reason that the
      Element hierarchy does NOT inherit from Image, the reason the Element
      class itself does not define an Image field, and the reason the Page
      class has an Image field. This approach significantly reduces memory
      costs and thus efficiency.
    </blockquote>
  <div>

</div>

            </td>

            <!-- begin side bar -->
            <td class="right" width="200">
<?php 
 include_once("sidebar.php");
?>

            </td>
            
            <!-- end side bar -->

          </tr>
      </table>
      </td>
    </tr>
    <tr>
      <td class="nopad">
<?php 
 include_once("footer.php");
?>
      </td>
    </tr>
  </tbody>
</table>


</body>
</html>
