<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <title>Conjecture - Modules</title>
  <meta http-equiv="content-type" content="text/html; charset=ISO-8859-15">
  <!-- link rel="icon" href="conjecture.ico" type="image/x-icon" -->
  <!-- link rel="shortcut icon" href="conjecture.ico" type="image/x-icon" -->
  <link rel="stylesheet" href="style.css" type="text/css" title="plain">
</head>
<body>
<table class="nopad" align="left" border="0" cellpadding="0"
 cellspacing="0" width="100%">
  <tbody>
    <tr>
      <td class="nopad">
<?php
 $mainvalue='Documentation';
 $subvalue='Modules';
 include_once("header.php");
?>
      </td>
    </tr>
    <tr>
      <td>

      <table class="nopad" align="left" border="0" cellpadding="0" cellspacing="0" width="100%">
        <tbody>
          <tr valign="top">
            <td class="content">

<div id="overview">
  <h1>An Overview of Modules</h1>
  <blockquote>

    <p>This documentation discusses the fundamental concept of a
    <em>OCR Module</em>, abbreviated simply as <em>Module</em>. It also
    discusses how Conjecture combines a collection of <A
    HREF="components.php">Components</A> into a Module, so an
    understanding of components is useful when reading this page
    (unfortunately, an understanding of modules is also useful when
    reading about components :-)

    <p>Conjecture provides support for analyzing, assessing,
    comparing, and modifying an unlimited number of OCRs. This is
    accomplished by defining an <em>OCR Module hierarchy</em>. Each
    class in this hierarchy represents a fully-functional OCR capable
    of taking an image as input and producing text as output. Each
    class in this hierarchy must deal with input image processing
    (noise removal, orientation adjustment, etc.), segmentation into
    glyphs (and words and lines and columns), identification of
    unicodes for each glyph, the production of properly formatted
    output, and everything else that goes into an OCR. However, these
    classes almost never do all of this work directly. Instead, they
    usually delegate responsibility for implementing individual issues
    to an <em>Implementation</em> of the <em>Component</em>
    representing that issue.

    <p>To clarify this more, suppose we want to create a new module.
    The name of this module is important both because it usually
    establishes the name of the class that will implement it, and also
    establishes the value to pass as the value of the command-line
    <code>-A</code> flag to indicate that this OCR module should be
    used to perform image-to-text translation. For our example module,
    we will choose the name 'test' (the value to provide to the
    <code>-A</code> flag), and use the class <code>TestModule</code>.

    <p>Now, it is possible to implement the entire image-to-text
    functionality using the <code>TestModule</code> class itself
    (along with any number of helper classes, of course). However, by
    implementing all of the code for the OCR in this fashion, it makes
    it difficult to experiment with alternative strategies for the
    various sub-components making up an OCR. The implementation
    becomes "rigid" and inflexible.

    <p>A much more flexible approach is to identify the various
    subcomponents, formalize them into <em>Component class
    hierarchies</em> (in which the root class provides an interface
    and subclasses represent alternative implementations), and have
    <code>TestModule</code> delegate responsibility for performing the
    various actions necessary to appropriate component implementation
    instances. This is the strategy choosen for the Conjecture
    framework.

    <p>Classes in the OCR Module hierarchy are usually very simple. So
    simple, in fact, that they can be automatically generated by the
    Conjecture infrastructure based on an entry in a special input
    file (see the discussion on <A
    HREF="#modules_file">Conjecture.modules</A>).

    <p>The OCR Module hierarchy has an abstract class called, not
    surprisingly, <code>OCRModule</code>, as its root. This class
    defines a special interface consisting of a collection of Factory
    Methods (methods responsible for creating objects). One Factory
    Method exits for each Component that Conjecture has identified and
    formalized. The <code>OCRModule</code> class also defines one
    field for each component. During creation, <code>OCRModule</code>
    uses the factory methods (redefined in subclasses to provide
    subclass-specific component implementation instances) to obtain
    instances to initialize these fields with.

    <p>Subclasses of the abstract <code>OCRModule</code> class
    represent concrete, fully-functional OCR implementations. They
    usually consist solely of definitions of the factory methods
    required by the <code>OCRModule</code> superclass. If the OCR in
    question does not need a particular Component, the associated
    factory method simply returns <code>NULL</code>. Each factory
    method specifies which Component subclass to create, and thus
    establishes which implementation of each Component is used by this
    particular OCR Module.

    <p>The Conjecture framework (specifically, the <code>Env</code>
    class) maintains a special <code>module</code> field that is
    initialized to an instance of a subclass of <code>OCRModule</code>
    based on the <code>-A</code> command-line flag. This field
    effectively acts as a container for a collection of Component
    subclass instances. Anywhere in the framework, if one wants to
    perform segmentation, they can ask for the
    <code>module</code>field of the (usually unique) <code>Env</code>
    instance. The resulting object has methods for obtaining the
    component implementtions associated with every component,
    including a <code>segment()</code> method that returns an instance
    of a subclass of <code>SegmentCompoment</code>. The
    <code>execute(Element*page)</code> method can then be invoked on
    this object in order to perform segmentation. All without having
    to know anything whatsoever about the actual implementation
    details. This design allows us to completely separate individual
    components from one another, making for a flexible plug-and-play
    paradigm.

    </ul>

    <div id="modules_file">
      <h2> The <code>config/Conjecture.modules</code> File </h2>
      <blockquote>

        <p> The <code>config/Conjecture.modules</code> file provides a
        description of every OCR Module currently supported by
        Conjecture. To add a new OCR Module, a user simply edits this
        file and adds another record in the same format as those
        already present. This file is expected to grow as more and
        more Modules are implemented, so we do not present a
        comphrensive summary here. Instead, we'll provide a small
        sample file and discuss the syntax for it:

        <pre>
  MODULE default CLASS DefaultModule
            COMPONENT Process  IMPLEMENTATION StandardProcess;
     extern COMPONENT Segment  IMPLEMENTATION GocrSegment;
     extern COMPONENT Identify IMPLEMENTATION GocrIdentify;
     extern COMPONENT Format   IMPLEMENTATION GocrFormat;

  MODULE gocr CLASS GocrModule
     extern COMPONENT Process  IMPLEMENTATION StandardProcess;
            COMPONENT Segment  IMPLEMENTATION GocrSegment;
            COMPONENT Identify IMPLEMENTATION GocrIdentify;
            COMPONENT Format   IMPLEMENTATION GocrFormat;

  MODULE ocrad CLASS OcradModule
            COMPONENT Process  IMPLEMENTATION OcradProcess;
     no     COMPONENT Segment;
     no     COMPONENT Identify;
     no     COMPONENT Format;

  MODULE wmh CLASS WmhModule
     extern COMPONENT Process  IMPLEMENTATION StandardProcess;
            COMPONENT Segment  IMPLEMENTATION WmhSegment;
            COMPONENT Identify IMPLEMENTATION WmhIdentify;
            COMPONENT Format   IMPLEMENTATION WmhFormat;
        </pre>

        <p> There are four Modules defined in this sample file, with the
        names 'default', 'gocr', 'ocrad' and 'wmh'. Note that the public
        name appears immediately after 'MODULE' keyword, and is not
        necessarily the same as the class name (which appears after the
        'CLASS' keyword). When invoking <code>conjecture</code> at the
        command line, the <code>-A<code> flag accepts a module name as a
        value.

        <p> Each OCR Module is fully described by specifying which class
        it will use to implement each Component. For example, GocrModule
        uses the StandardProcess subclass of ProcessComponent to implement
        the 'process' issue, and uses Gocr-specific sublcasses of
        SegmentComponent, IdentifyComponent and FormatComponent to deal
        with segmentation, identification and formatting respectively.

      </blockquote>
    </div>

    <div id="ocrgen">
      <h2> The <code>bin/ocrgen</code> Script </H3>
      <blockquote>

        <p>The <code>ocrgen</code> script reads the
        <code>Conjecture.modules</code> file and verifies the existence of an
        appropriate subdirectory of <code>src/custom</code>, and source/header
        files for the Module and Component classes provided by the Module.

        <p> ... more here ...
      </blockquote>
    </div>

  </blockquote>
</div>

<div id="creating">
  <h1>Creating a new OCR Module</h1>
  <blockquote>

  <p><b>NOTE:</b> This is a very brief description of a really important
   topic that needs to be much more fully addressed.  Please contribute
   documentation if you have ideas for how to explain the process better.

   <p>Before making a module of your own, it is useful to understand
   how to run existing modules on images, so you should read the
   <A HREF="usage.php">Usage</A> documentation before preceeding with
   this discussion.  This discussion assumes that Conjecture is
   installed and is working correctly.  To determine this, try:

   <pre>
    prompt% cd $CONJECTURE
    prompt% make verify
   </pre>

   <p> If the above does not produce a table consisting of a whole
   bunch of plus signs ('+'), something is wrong. Inform us on the <A
   HREF="http://lists.corollarium.com/listinfo.cgi/conjecture-devel-corollarium.com">developers
   mailing list</A>.

   <p>Here is the way you should create your first OCR Module.  It
   differs somewhat from how you would create subsequent module (once
   you become more familiar with the Conjecture architecture), but
   provides a quick means of getting something up-and-running
   immediately.

  <ol>

    <li> Establish a name for your module. We'll use the name
    <code>test</code>, but for your module, I'd suggest your initials.
    For example, my first OCR module is <code>wmh</code>.

    <li> Edit <code>$CONJECTURE/config/Conjecture.modules</code>
    (see the <A HREF="#moudles_file">discussion</A>) by adding
    a new record that looks like this (or uncommenting the
    record already present in the file for exactly this purpose):

  <pre>
   MODULE test CLASS TestModule EXTENDS GocrModule
     extern COMPONENT Process  IMPLEMENTATION StandardProcess;
     new    COMPONENT Segment  IMPLEMENTATION TestSegment     EXTENDS GocrSegment;
     new    COMPONENT Identify IMPLEMENTATION TestIdentify    EXTENDS GocrIdentify;
     new    COMPONENT Format   IMPLEMENTATION TestFormat      EXTENDS GocrFormat;
  </pre>

  <p>The above is informing Conjecture that you want an OCR module
  named <code>test</code>, with module class <code>TestModule</code>.
  Furthermore, it says that you want to use the (pre-existing)
  <code>StandardProcess</code> implementation for the
  <code>Process</code> component (which means you do not need to
  implement it at all). It also says that this new module will be
  providing its own implementations for the <code>Segment</code>,
  <code>Identify</code> and <code>Format</code> components. However,
  it also indicates that each of these new implementations will be
  based on the corresponding GOCR implementation (that is, that the
  classes representing the implementations will inherit from the gocr
  implementation classes).  This last fact is important, because it
  allows us to create a new Module that will be able to perform
  image-to-text identification immediately, after which you can start
  incrementally replacing or augmenting gocr functionality as you
  desire.

  <p>Note that currently Conjecture has the most support for
  interacting with gocr, but support for other third-party ocrs will
  be added when we (including you!) do it. If you are remotely
  familiar with the architecture of any existing open-source OCR,
  please let us know on the <A
  HREF="http://lists.corollarium.com/listinfo.cgi/conjecture-devel-corollarium.com">developers
  mailing list</A>.

  <li> Once you've edited the <code>Conjecture.config</code> file,
  do the following:

  <pre>
   prompt% cd $CONJECTURE/src/custom
   prompt% ls
  </pre>

  <p>The <code>src/custom</code> directory is where all the
  module-specific code (primarily, implementations of components)
  reside. There is a subdirectory for each OCR module. Note that your
  new module, <code>test</code>, is not yet there. The
  <code>abstract</code> subdir is special (it isn't a module, but
  instead contains the abstract superclasses of the Module and
  Component hierarchies). The <code>SomeName</code> files are also
  special - they are templates for any new classes you might want to
  create.

  <li> Now do the following:

  <pre>
   prompt% make modules
   prompt% ls
   prompt% ls test
  </pre>

  <p>This causes Conjecture to invoke the special <code>ocrgen</code>
  script, which is responsible for parsing the
  <code>$CONJECTURE/config/Conjecture.modules</code> file (the one you
  just edited) and ensuring that C++ classes exist corresponding to
  the information present in that file. Most of the classes specified
  by the file already exist, but those associated with your new
  <code>test</code> module do not, so after execution, you will note
  that a <code>test</code> subdir exists, and contains a variety of
  classes. Furthermore, the classes already contain an entirely
  functional implementation (Conjecture has automatically generated
  appropriate method signatures and default method implementations).

  <li> Now we compile the new code:

  <pre>
   prompt% make
  </pre>

  <p>Note that typing <code>make</code> in the <code>src/custom</code>
  directory ended up performing a <code>make</code> in
  <code>src</code>. This is intentional. The makefiles in
  subdirectories of <code>src</code> usually just delegate those
  targets to the makefile in <code>src</code>, which is smart enough
  to compile every source file in every subdirectory, and will thus
  detect the new source code for the <code>test</code> module.

  <li> Now we test your new module:

  <pre>
   prompt% cd ..          # directory $CONJECTURE/src
   prompt% make test.pnm  # creates a symlink to a test image for convenience
   prompt% conjecture -i test.pnm -o test.ocr -M test
   prompt% cat test.ocr
  </pre>

  <p>Note that your module is producing the same output as the 'gocr'
  module. Which shouldn't be surprising since all of the components in
  your module are currently inheriting all functionality from their
  parent 'gocr' components.

  <li> Now you can start modifying your module, by editing the
  component classes and adding new code.

  <pre>
   % cd custom/test
   % ls
  </pre>

  <p> Edit the <code>TestIdentify.cc</code> file and insert the
  following line as the first line in the
  <code>TestIdentify::execute</code> method.

  <pre>
    cerr << "NOTE: Here in TestIdentify::execute" << endl;
  </pre>

  <p>Then compile and run again:

  <pre>
   # This command can be executed in any directory and will compile
   # the source code properly, because all Makefiles delegate to
   # $CONJECTURE/src
   prompt% make    
 
   # This command will only work in a directory that has a
   # test.pnm file (in our example so far, $CONJECTURE/src)
   prompt% conjecture -i test.pnm -o test.ocr -M test
  </pre>

  <p>Note that running the program now prints out the line you added
  to your TestIdentify component, demonstrating that Conjecture really
  is executing your module.

  <li> Now comes the hard part. You start adding code to some or all
  of the component classes in the <code>custom/test</code> directory.
  These changes either build up on what gocr is doing, or entirely
  replacing what it is doing. Of course, building on top of gocr
  requires knowledge on your part of the architecture and
  implementation details of gocr. And no matter what, you will need to
  understand the basics of the Element hierarchy, provided by
  Conjecture. This hierarchy sub-divides an image into semantically
  meaningful sub-regions (pages, regions, lines, words and glyphs),
  and must be modified in order to satisfiy the post-conditions of
  each component. Which also implies that you need to understand the
  pre and post conditions of each <A
  HREF="components.php#existing">component</A>, and ensure that your
  component implementations satisfy the post-conditions upon
  completion (your component implementations can assume that the
  pre-conditions hold).

  <li> The above was just an example. If you used the name
  <code>test</code>, please do not add it into the repository (this
  howto won't be as useful if you do), do not commit
  <code>Components.modules</code> with the test record uncommented,
  and do not commit <code>Modules.cc</code> if it assumes the
  existence of a TestModule class. On the other hand, once you have
  created a new, more appropriately named, module, you are very much
  encouraged to commit it (along with the new
  <code>Components.modules</code> and <code>Modules.cc</code> files).


 </ol>

 <p>The above process is useful if you want to improve the GOCR
 implementation. However, if you are more comfortable with the inner
 workings of some other third-party open-source OCR, you would change
 the record you added to <code>Conjecture.modules</code>
 appropriately. However, in order to start building on top of some
 other third-party OCR, Conjecture must first be extended to support
 it. If you are interested in extending an OCR that is not currently
 supported by Conjecture, please let us know on the <A
 HREF="http://lists.corollarium.com/listinfo.cgi/conjecture-devel-corollarium.com">developers
 mailing list</A>.


 <!--
   <p> Below is a more general description of what to do when
    creating a new Module.

   <ol>

    <li> Determine whether you will be building on top of an existing
    OCR, or performing everything yourself. In order to get something
    working quickly, let's assume you are going to be building on top
    of GOCR. 

    <li> Determine which Components you will be dealing with
    personally (that is, that you will be providing implemenations
    for), and which ones you will be reusing some other component
    implemention for. Let's assume that you will eventually be dealing
    with all aspects of image-to-text translation yourself. This means
    you will want to introduce new implementations of all the
    components. However, since your new component implementations can
    inherit from existing component implementations, we can get the
    best of both worlds - an existing implementation for now, which
    can be replaced or augmented incrementally when you find the time.

    <li> Having established the basics, you will want to edit


  </ol>

 -->

  </blockquote>
</div>

<div id="existing">
  <h1>OCR Modules Currently Available in Conjecture</h1>
  <blockquote>

    <p>The current class diagram for the OCRModule hierarchy is:

      <div id="uml-modules-hier">
         <?php include_once("uml/modules-hier.map"); ?>
         <center><img src="uml/modules-hier.jpg" usemap="#modules-hier"></center>
      </div>

    <ul>

      <li> The <code>DefaultModule</code> class describes the default
      Conjecture implementation, and will represent the current "best"
      overall strategy.

      <li> The <code>GocrModule</code> class relies on the GOCR library,
      and provides facilities for having Conjecture data-structures like
      Page, Glyph, etc. interact with underlying GOCR data-structures like
      Job and box.

      <li> The <code>OcradModule</code> provides access to the Ocrad OCR,
      but currently via a sub-process execution (as opposed to linking the
      Ocrad source code into Conjecture). This will be changed as
      Conjecture evolves.

      <li> The <code>WmhModule</code> is an example of an experimental
      module. It represents Wade's first attempt at various parts of
      the OCR problem (currently focusing on identification). It
      relies on the GOCR module to provide segmentation, but provides
      its own identification and formatting algorithms.
    </ul>

    <div id="uml-modules-pub">
        <?php include_once("uml/modules-pub.map"); ?>
       <center><img src="uml/modules-pub.jpg" usemap="#modules-pub"></center>
    </div>

  </blockquote>
</div>

            </td>

            <!-- begin side bar -->
            <td class="right" width="200">
<?php
 include_once("sidebar.php");
?>

            </td>

            <!-- end side bar -->

          </tr>
      </table>
      </td>
    </tr>
    <tr>
      <td class="nopad">
<?php
 include_once("footer.php");
?>
      </td>
    </tr>
  </tbody>
</table>


</body>
</html>
