# include <iostream>
# include "OCRModule.h"  

namespace Conjecture {

  OCRModule::OCRModule() {
    // We CANNOT call 'init' within this constructor because
    // pure-virtual methods are not yet fully "available" within
    // constructors. After creation, we must call 'init'. This is NOT
    // a desirable feature and violates the mandate that all classes
    // be fully initialized after the constructor has executed.
    //
    // FUTURE FIX: Can this be resolved, so that the OCRModule
    // class IS fully initialized when the constructor returns?? One
    // "solution" would be to call 'init' within the child
    // constructors. However, that requires clients who are defining
    // OCRModule subclasses to remember to do so (error prone).
    //
    // Currently, the OCRModule subclass instance is created in
    // exactly one place (the Env::Env constructor). Within that
    // method, we ensure that 'init' is called immediately after the
    // constructor has returned. The danger comes if we create a
    // OCRModule someplace else, and do not realize that 'init'
    // must be callled afterwards.
  }

  void OCRModule::init() {
    this->processIs( this->createProcessComponent() );
    this->segmentIs( this->createSegmentComponent() );
    this->identifyIs( this->createIdentifyComponent() );
    this->formatIs( this->createFormatComponent() );
  }

};
