// C++ Dependencies
using namespace std;
# include <iostream>
# include <fstream>
# include "GocrFormat.h"
# include "Page.h"
# include "Glyph.h"
# include "Line.h"
# include "GocrModule.h"

// C Dependencies
# include "gocr.h"     // Job
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
# include "pgm2asc.h" // pgm2asc, getTextLine

extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);
// FUTURE FIX: Most of the above are NOT needed in this class!!

namespace Conjecture {
  GocrFormat::GocrFormat() {
  }

  bool GocrFormat::execute(Page* page) {
    //DEBUG: cerr << "**** HERE in GocrFormat::format" << endl;

    bool result = false;

    // Obtain the outfile
    const Env* env = page->env();
    const string& outfile = env->outfile();
    
    // Obtain the module
    const GocrModule* module = dynamic_cast<const GocrModule*>(env->module());
    if ( module == NULL ) {
	throw string("IncompatibleModuleException: GocrFormat requires the GocrModule module");
    }

    // Obtain the ubiquitous Job structure
    Job* job = module->job();

    // Open the desired file.
    ofstream os(outfile.c_str());
    if ( !os ) {
      // FUTURE FIX: Report the reason for the failure!  How?
      cerr << "ERROR: Failed to open " << outfile << " for writing" << endl;
      result = false;

    } else {
      // ******************************************************  
      store_boxtree_lines( job->cfg.mode );
      progress(90,0); /* progress is only estimated */
      
      // ******************************************************  
      // write out30.pgm
      if( job->cfg.verbose&32 ) debug_img("out30",job,2+4);

      progress(100,0); /* progress is only estimated */
      
      // ******************************************************  
      // Collesce characters into lines.
      int lc = 0;
      const char* line = getTextLine(lc++);
      int format = job->cfg.out_format;
      while (line) {
        os << line;
        if (format == HTML ) { os << "<br/>"; }
        if ( format != XML ) { os << endl; }
        line = getTextLine(lc++);
      }
      free_textlines();

      // FUTURE FIX: If 'outfile' is '-', does closing it close 'cout'
      // in addition to 'os? If so, we should special-case the close
      // (don't close if 'outfile' is '-'!
      os.close();

      result = true;
    }
    return result;

  }
};

