// C++ Dependencies
using namespace std;
# include <iostream>
# include "GocrIdentify.h"
# include "GocrModule.h"
# include "Page.h"
# include "Glyph.h"
# include "Line.h"

// C Dependencies
# include "gocr.h"     // Job
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
# include "pgm2asc.h" // pgm2asc, getTextLine


extern wchar_t whatletter(Box *box1, int cs, int nrun);

extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);
// FUTURE FIX: Most of the above are NOT needed in this class!!

namespace Conjecture {
  GocrIdentify::GocrIdentify() {
    // Default Constructor
  }

  bool GocrIdentify::execute(Page* page) {
    //DEBUG: cerr << "**** HERE in GocrIdentify::identify" << endl;
    // Obtain the env
    const Env* env = page->env();

    // Obtain the module
    const GocrModule* module = dynamic_cast<const GocrModule*>(env->module());
    if ( module == NULL ) {
	throw string("IncompatibleModuleException: GocrFormat requires the GocrModule module");
    }

    // Obtain the ubiquitous Job structure
    Job* job = module->job();

    // Set up some local vars
    int  mode    = job->cfg.mode;
    pix* pp      = &(job->src.p);
    bool result  = true;

    // ******************************************************
    // Perform the actual character recognition.
    char_recognition( pp, job->cfg.mode);
    progress(60,0); /* progress is only estimated */

    // ******************************************************    
    if ( adjust_text_lines( pp, mode ) ) {
      //DEBUG: cerr << "NOTE: Sorting lines and recalling char_recognition!" << endl;
      /* correct using chars */
      /* may be, characters/pictures have changed line number */
      list_sort(&(job->res.boxlist), sort_box_func);
      // 2nd recognition call if lines are adjusted
      char_recognition( pp, job->cfg.mode);
    }

    // ******************************************************
    // write out20.pgm - mark lines + boxes
    if(job->cfg.verbose&32) debug_img("out20",job,1+4+8);

    // ******************************************************
    compare_unknown_with_known_chars( pp, job->cfg.mode);
    progress(70,0); /* progress is only estimated */

    // ******************************************************
    try_to_divide_boxes( pp, job->cfg.mode);
    progress(80,0); /* progress is only estimated */

    // ******************************************************
    // list output - for debugging
    if( job->cfg.verbose&6 ) output_list(job);

    // ******************************************************
    list_insert_spaces( pp , job );

    // ******************************************************
    // proof difficult chars Il1 by context view
    if(!(job->cfg.mode&32)) context_correction( job );

    return result;
  }
};

