# ifndef Conjecture_GocrModule_h
# define Conjecture_GocrModule_h Conjecture_GocrModule_h

// Insert SVN header substitution variable here
// Created by wade
// Class Champion: Unassigned

// C++ Dependencies
# include "OCRModule.h"  // parent

// C Dependencies (temporary)
# include "gocr.h"  // job_t, job_init

namespace Conjecture {

    /** ****************************************************************
     * \class GocrModule
     *
     * \brief Uses the Gocr Strategies for all algorithms
     *
     * Every algorithm in the Conjecture framework has an equivalent in
     * GOCR (or the algorithm is subsumed by a higher-level algorithm
     * in GOCR). This factory returns back the Gocr Strategies for all
     * algorithms.
     *
     ******************************************************************/

    class GocrModule : public OCRModule {
      public:
        GocrModule();

        virtual ProcessComponent *  createProcessComponent()   const;
        virtual SegmentComponent *  createSegmentComponent()   const;
        virtual IdentifyComponent*  createIdentifyComponent()  const;
        virtual FormatComponent  *  createFormatComponent()    const;

        virtual void processArgs(int argc, const char** argv, Env* env = NULL);

        // NOTE: All of these are public (and the getter isn't
        // read-only) due to the convoluted nature of underlying
        // 'gocr' code. It is hoped that 'gocr' will clean up its
        // reliance on Job, after which we can start enforcing a
        // cleaner design in these wrapper classes.
        inline Job*       job() const     { return this->_job; }
        inline Job* &     jobRef()        { return this->_job; }
        inline void       jobIs(Job* job) { this->_job = job;  }

      protected:
        
      private:
        Job*            _job;
  };
}

# endif // Conjecture_GocrModule_h
