using namespace std;
# include <iostream>
# include <iomanip>
# include <sstream>
# include "OptIm.h"
# include "Image.h"
# include "BitOverlay.h"  // Regions
# include "BitInfo.h" // various

// Insert SVN header substitution variable here
// Created by wade
// Class Champion: wade

// Include

namespace Conjecture {
    //     OptIm::OptIm(const Image* image, const ImageArgs& args) {
    //         // We create an OptIm instance from the given Image. If the
    //         // size of the image exceeds MaxPixels pixels in either
    //         // dimension, that dimension is truncated to MaxPixels. Thus,
    //         // this method grabs the top left corner of larger images,
    //         // always.
    //         vector<u4> rows;
    //         u1 szx = BitImage(image,args,rows);
    //         this->init(rows,szx);
    //     }

    OptIm::OptIm(const Image* image, const ImageArgs& args) {
        PixelMatrix& rows = this->rowsRef();
        u2 width = image->size().x;
        u2 height = image->size().y;
        if ( width > PixelLine::MaxPixels ) { width = PixelLine::MaxPixels; }
        if ( height > PixelLine::MaxPixels ) { height = PixelLine::MaxPixels; }

        for (u2 y = 0; y < height; ++y) {
            // We obtain the one-bit pixel values for this row of the image
            u2 pixels = 0;
            for (u2 x = 0; x < width; ++x) {
                int val = args.apply(image,Coord(x,y));
                if ( val == ImageArgs::MinVal ) {
                    pixels |= (1<<(width-1-x));
                }
            }
            rows.push_back(PixelLine(pixels));
        }
        this->widthIs(width);
        this->updatedIs(false);
    }

    OptIm::OptIm(const vector<unsigned int>& inrows, u1 width) {
        // Create rows of PixelLines from 'inrows'
        PixelMatrix& rows = this->rowsRef();
        for (unsigned int i = 0; i < inrows.size(); ++i) {
            rows.push_back(PixelLine(inrows[i]));
        }
        this->widthIs(width);
        this->updatedIs(false);
    }

    void OptIm::updateMeta() {
        if ( ! this->updated() ) {
            
            u1 height = this->height();
            u1 width  = this->width();
            
            // Compute metadata for each PixelLine in rows
            PixelMatrix& rows = this->rowsRef();
            for (PixelMatrix::iterator it = rows.begin(); it != rows.end(); ++it) {
                it->updateMeta();
            }
            
            // Create columns of PixelsLines from 'rows', and compute
            // metadata for each. We are storing the right-most column in
            // index 0 (this gives bit ordering). FUTURE FIX: Why did I
            // choose bit ordering? Image ordering is probably more
            // intuitive??
            PixelMatrix& cols = this->colsRef();
            for (int c = width-1; c >= 0; --c) {
                unsigned int p = 0;
                for (unsigned int r = 0; r < height; ++r) {
                    if ( rows[r].get(c) ) { p |= (1<<r); }
                }
                cols.push_back(PixelLine(p));
                cols[cols.size()-1].updateMeta();
            }
            //DEBUG: cerr << this->rawstr(cols,'#','.',height) << endl;
            this->updatedIs(true);
        }
    }

    // Interface

    std::string OptIm::str(const string& title, int valign, char on, char off) const { // , u1 size) const {
        stringstream s;
        u2 size = this->width();
        
        // Obtain the columns
        const PixelMatrix& cols = this->cols();

        // Add the title
        if ( title.size() > 0 ) {
            s << "   ";
            if ( title.size() > size ) {
                s << title.substr(0,size-1) + string("$");
            } else {
                s << title << string(size - title.size(), ' ');
            }
            s << endl;
        }

        // Add the vertical alignment
        if ( valign > 0 ) {
            string pad(size+3, ' ');
            for (int i = 0; i < valign; ++i) {
                s << pad << endl;
            }
        }
        
        // Print out an initial line representing column hole counts per
        // row. Since PixelLine::str prints out row holes in 2 digits,
        // then space, then row data, then space, then black count in 2
        // digits, we use the same format for this header (and the footer
        // later).
        s << "  " << " ";
        if ( cols.size() > 0 ) {
            for (PixelMatrix::const_iterator it = cols.begin(); it != cols.end(); ++it) {
                u1 holes = it->holes();
                char h;
                if ( holes > 9 ) {
                    h = char( (holes-9)-(int('a')) );
                } else {
                    h = char( int('0')+holes);
                }
                s << h;
            }
        } else {
            s << string(size,' ');
        }
        s << endl;
        
        // Now print the rows
        s << this->rawstr(this->rows(), on,off, size);
        
        // And now print a footer consisting of the column black counts
        if ( cols.size() > 0 ) {
            // We don't output a line at all if no cols
            s << "  " << " ";
            for (PixelMatrix::const_iterator it = cols.begin(); it != cols.end(); ++it) {
                u1 black = it->black();
                char h;
                if ( black > 9 ) {
                    h = char( (int('a')) + (black-10) );
                } else {
                    h = char( int('0')+black);
                }
                s << h;
            }
            s << endl;
        } else {
            // ?? s << "...";
        }
        
        // Return result
        return s.str();
    }

    std::string OptIm::rawstr(const PixelMatrix& vec, char on, char off, u1 size) const {
        stringstream s;
        for (PixelMatrix::const_iterator it = vec.begin(); it != vec.end(); ++it) {
            s << it->str(on,off,size) << endl;
        }
        return s.str();
    }

    bool OptIm::crop(int level) {
        // for now we limit 'level' to avoid crazy high values and the
        // special case code it would necessitate.
        if ( level > 3 ) { level = 3; } 
        return this->crop(level,level,level,level);
    }

    bool OptIm::crop(int topthick, int bottomthick, int leftthick, int rightthick) {
        return this->crop(topthick,bottomthick, leftthick,rightthick,topthick,bottomthick, leftthick,rightthick);
    }

    bool OptIm::crop(int topthick, int bottomthick, int leftthick, int rightthick, int& topref, int& bottomref, int& leftref, int& rightref) {
        bool result = false;

        // NOTE: Possible future extension:
        //   - currently, if there is a single failure to match
        //     pixel to adjacent pixel within a row/column, we do
        //     not make ANY deletions.  For this reason, 
        //     crop(1,1,3,3) on the left image produces the right:
        //                           
        //         ...######.          1222221  
        //         ..##.####.        0 ####### 7
        //         .##...##..        1 #...##. 3
        //         .##...##..        1 #...##. 3
        //         .##...##..        1 #...##. 3
        //         ...#.##...        0 .####.. 4
        //         ...###....        0 .###... 3
        //         .##.......        0 ####... 4
        //         ..##.####.        0 ####### 7
        //         ..###.###.        0 ####### 7
        //         .##.....##        1 #.....# 2
        //         ##......#.        1 #.....# 2
        //         .###..#.#.        1 ##..### 5
        //         ..#.###...        0 #####.. 5
        //                             b877976  
        //
        //     Note that the top right part of the region was NOT
        //     cropped because the bottom right region could not be
        //     cropped. We could, however, allow individual contiguous
        //     regions to be cropped independently of one another
        //     within a row/column. This was in fact what was done in
        //     the initial implementation. However, doing so poses
        //     additional subtleties that need to be addressed (this
        //     wasn't realized until the existing implementation was
        //     done). To see the problem, suppose we do strip the lone
        //     pixel at the top of column 6 (but not the region at the
        //     bottom of column 6, of course). Now, suppose we
        //     continue by stripping column 5. The top part, from rows
        //     0-3, can be stripped safely. And if we use the existing
        //     algorithm, the regions from row 7-8 of column 5 can
        //     also be stripped because the current code is only
        //     ensuring that we don't introduce holes in the vertical
        //     direction when processing columns. However, in this
        //     situation we should disallow stripping column 5 rows
        //     7-8 because doing so introduces holes in the
        //     *horizontal* direction. Detecting such hole
        //     introduction in the orthogonal direction may be
        //     difficult. Howeever, an alternative strategy might be
        //     to simply mark the entire region from row 7-11 as
        //     "to-be-ignored" when processing column 5 because that
        //     region wasn't stripped during column 6. I suspect that
        //     will produce fairly good results.

        PixelMatrix& rows = this->rowsRef();
        u1 height = this->height();
        u1 width  = this->width();
        u1 ymax   = height - 1;
        u1 todelete[PixelLine::MaxPixels];
        u1 ndel = 0;

        int debug = 0;
        DBG(1, this->str());
        
        // Top of image
        int topdel = -1;
        for (int dy = 0; dy < topthick; ++dy) {
            PixelLine& top   = rows[dy];
            PixelLine& tnext = rows[dy+1];

            DBG(1,"Top: " << dy);
            int  count   = 0;
            bool inhole  = true;
            int  cts     = 0;

            ndel = 0;
            // subtlety: bits beyond bit position 'width-1' are always
            // 0, so 'top.get(width)' always returns 0. We iterate up
            // to 'bit==width' in order to "flush" a contiguous region
            // that ends at the left border.
            for (int bit = 0; bit <= width; ++bit ) {
                // right-to-left scanning - PixelLine is bit-coord!
                if ( top.get(bit) ) {
                    // The bit is on. If the same bit in the next row is
                    // also on, and clearing the current bit will not
                    // introduce holes where there were no holes
                    // before, then we clear it.
                    if ( inhole && tnext.get(bit) ) {
                        // We extend the contiguous region to be deleted
                        ++cts;
                    } else {
                        // The current bit is not shadowed by the
                        // corresponding bit in the next row, so the
                        // entire current contiguous region cannot be
                        // deleted.
                        inhole = false;
                        cts = 0;
                    }
                    ++count;
                    DBG(1, "  x=" << setw(2) << (width-1-bit) << " " << top.get(bit) << " vs " << tnext.get(bit) << " cts " << cts << " count " << count);
                } else {
                    if ( cts > 0 ) {
                        for (int b = bit-cts; b < bit; ++b) {
                            todelete[ndel++] = b;
                        }
                        cts = 0;
                    }
                    inhole = true;
                }
            }
            if ( count > ndel ) {
                // After region deletion, there are more pixels on than
                // are marked as eligible for deletion, so we cannot
                // delete this row (we can delete all rows less than it,
                // and stop further processing).
                topdel = dy;
                break;
            } else {
                for (int i = 0; i < ndel; ++i) {
                    int b  = todelete[i];
                    int dx = width-1-b;
                    DBG(1, "    clearing <" << dx << "," << dy << ">");
                    top.rawclear(b);
                }
            }
        }
        if ( topdel==-1 ) { topdel = topthick; }

        // Bottom of image
        int bottomdel = -1;
        for (int dy = 0; dy < bottomthick; ++dy) {
            PixelLine& bottom = rows[ymax-dy];
            PixelLine& bprev  = rows[ymax-dy-1];

            DBG(1,"Botom: " << dy);
            int  count  = 0;
            bool inhole = true;
            int  cts    = 0;

            ndel = 0;
            // See note in top-of-image for why <= width!
            for (int bit = 0; bit <= width; ++bit ) {
                // right-to-left scanning - PixelLine is bit-coord!
                if ( bottom.get(bit) ) {
                    // The bit is on. If the same bit in the prev row
                    // is also on, and clearing the current bit will
                    // not introduce holes where there were no holes
                    // before, then we clear it.
                    if ( inhole && bprev.get(bit) ) {
                        ++cts;
                    } else {
                        // The current bit is not shadowed by the
                        // corresponding bit in the prev row, so the
                        // entire current contiguous region cannot be
                        // deleted.
                        inhole = false;
                        cts = 0;
                    }
                    ++count;
                    DBG(1, "  x=" << setw(2) << (width-1-bit) << " " << bottom.get(bit) << " vs " << bprev.get(bit) << " cts " << cts << " count " << count);
                } else {
                    if ( cts > 0 ) {
                        for (int b = bit-cts; b < bit; ++b) {
                            todelete[ndel++] = b;
                        }
                        cts = 0;
                    }
                    inhole = true;
                }
            }
            if ( count > ndel ) {
                bottomdel = dy;
                break;
            } else {
                for (int i = 0; i < ndel; ++i) {
                    int b  = todelete[i];
                    int dx = width-1-b;
                    DBG(1, "    clearing <" << dx << "," << height-1-dy << ">");
                    bottom.rawclear(b);
                }
            }
        }
        if ( bottomdel==-1 ) { bottomdel = bottomthick; }

        // Left of image
        int leftdel = -1;
        for (int dx = 0; dx < leftthick; ++dx) {

            int  count  = 0;
            bool inhole = true;
            int  cts    = 0;
            ndel        = 0;

            // bit position of image x coord dx
            u1 bpos = width-1-dx; 

            DBG(1,"Left: " << dx);
            for (int y = 0; y < height; ++y) {
                if ( rows[y].get(bpos) ) {
                    // determine if the pixel to the right (bpos-1) is
                    // also set.
                    if ( inhole && rows[y].get(bpos-1) ) {
                        // Pixel matches adj. in prev line
                        ++cts;
                    } else {
                        inhole = false;
                        cts = 0;
                    }
                    ++count;
                    DBG(1, "  y=" << setw(2) << y << " " << rows[y].get(bpos) << " vs " << rows[y].get(bpos-1) << " cts " << cts << " count " << count);
                } else {
                    // Pixel isn't set
                    if ( cts > 0 ) { 
                        // CODE TANGLE #2
                        for (int dy = y-cts; dy < y; ++dy) { 
                            todelete[ndel++] = dy;
                        }
                        cts = 0;
                    }
                    inhole = true;
                }
            }
            if ( cts > 0 ) { 
                // CODE TANGLE #2 (special case y==height)
                for (int dy = height-cts; dy < height; ++dy) { 
                    todelete[ndel++] = dy;
                }
                cts = 0;
            }
            if ( count > ndel ) {
                leftdel = dx;
                break;
            } else {
                for (int i = 0; i < ndel; ++i) {
                    int dy = todelete[i];
                    DBG(1, "    clearing <" << dx << "," << dy << ">");
                    rows[dy].rawclear(bpos);
                }
            }
        }
        if (leftdel==-1) { leftdel = leftthick; }

        // Right of image
        int rightdel = -1;
        for (int dx = 0; dx < rightthick; ++dx) {

            int  count  = 0;
            bool inhole = true;
            int  cts    = 0;
            ndel        = 0;

            u1   bpos   = dx;

            DBG(1,"Right: " << (width-1-dx));
            for (int y = 0; y < height; ++y) {
                if ( rows[y].get(bpos) ) {
                    // determine if the pixel to the left (b+1) is
                    // also set.
                    if ( inhole && rows[y].get(bpos+1) ) {
                        // Pixel matches adj. in next line
                        ++cts;
                    } else {
                        inhole = false;
                        cts = 0;
                    }
                    ++count;
                    DBG(1, "  y=" << setw(2) << y << " " << rows[y].get(bpos) << " vs " << rows[y].get(bpos+1) << " cts " << cts << " count " << count);
                } else {
                    // Pixel isn't set
                    if ( cts > 0 ) { 
                        // CODE TANGLE #3
                        for (int dy = y-cts; dy < y; ++dy) { 
                            todelete[ndel++] = dy;
                        }
                        cts = 0;
                    }
                    inhole = true;
                }
            }
            if ( cts > 0 ) { 
                // CODE TANGLE #3
                for (int dy = height-cts; dy < height; ++dy) { 
                    todelete[ndel++] = dy;
                }
                cts = 0;
            }
            if (count > ndel) {
                rightdel = dx;
                break;
            } else {
                for (int i = 0; i < ndel; ++i) {
                    int dy = todelete[i];
                    DBG(1, "    clearing <" << width-1-dx << "," << dy << ">");
                    rows[dy].rawclear(bpos);
                }
            }
        }
        if (rightdel==-1) { rightdel = rightthick; }

        // Now perform the deletion itself.
        DBG(1,  "Deleting " << topdel << " top " << bottomdel << " bottom " << leftdel << " left " << rightdel << " right");

        // We shift PixelLine's down 0 or more spots,
        // shfiting pixels right 0 or more.
        int newwidth  = width  - leftdel - rightdel;
        if ( newwidth < 1 ) { newwidth = 1; }
        int newheight = height - topdel - bottomdel;
        if ( newheight < 1 ) { newheight = 1; }
        
        for (int y = 0; y < newheight; ++y) {
            rows[y].assign(rows[y+topdel], rightdel);
        }
        while (rows.size() > (u1)newheight) {
            rows.pop_back();
        }
        this->widthIs(newwidth);

        topref    += topdel;
        bottomref += bottomdel;
        leftref   += leftdel;
        rightref  += rightdel;

        result = ( topdel || bottomdel || leftdel || rightdel ) ? true : false;
        return result;
    }

    bool OptIm::removeBottomSerifs() {
        // Bottom serifs can mess up recognition because they can make
        // a glyph that should have a down cup instead look like it
        // has a whole, either because thresholding has melted nearby
        // serifs together, or because the serifs are very close to
        // being melted and the standard 101-->111 filter melts them
        // after the fact.
        //
        //                    101-->111           crop
        //                     filter           2,2,2,2
        //
        //  .###.###...      .#######...        0011110   
        //  .#######...      .#######...      0 ######. 6 
        //  ..##...##..      ..##...##..      1 ##...## 4 
        //  ..##....#..      ..##....#..      1 ##....# 3 
        //  ..##....#.. ---> ..##....#.. ---> 1 ##....# 3 
        //  ..##....#..      ..##....#..      1 ##....# 3 
        //  ..##....#..      ..##....#..      1 ##....# 3 
        //  .###....##.      .###....##.      1 ##....# 3 
        //  #####.#####      ###########      0 ####### 7 
        //                                      8822237   
        //
        // Note that the melting has made the image look much more
        // like an 'o' than a 'n'.
        //
        // This method helps address the "close-to-melted" serif
        // problem shown above, in which the 101-->111 filter melts
        // serifs. 
        //
        // Most bottom serifs are relative easy to detect. If the
        // bottom row's pixel vector is a superset of the row above
        // it, it indicates the possibility of a serif. For example,
        //          '.###....##.'
        //        & '#####.#####'
        //        = '.###....##.'
        //
        // so we can replace bottom with bottom-2 (which avoids the
        // melting issue). However, using this test:
        //
        //   (bottom & second-from-bottom) == second-from-bottom
        //
        // can identify some things that aren't serifs. For
        // example, an 'o' might look like this:
        //            '.###....##.'
        //          & '###########'
        //          = '.###....##.'
        // but replacing bottom with bottom-2 in this situation
        // would make the 'o' look like an 'n' (the opposite
        // problem).
        //
        // By ensuring that both lines have the same number of
        // "1-regions", we avoid this problem.
        //
        // ---------------------------------------------------------
        //
        // Although the above "close-to-melted" strategy works fairly
        // well, it is also important to be able to handle
        // already-melted serifs. Not only because a particular
        // thresholding value may melt serifs from the get-go, but
        // also because the above "close-to-melted" heuristic does not
        // always work, and after a 101->111 filter pass a
        // "close-to-melted" glyph in which serifs were not detected
        // becomes a mleted-serif glyph.
        //
        // For example,
        //
        //       ..##.......          ..##.......
        //       .###.......          .###.......
        //       .###.......          .###.......
        //       ..##.......          ..##.......
        //       ..##.......          ..##.......
        //       ..##.......          ..##.......
        //       ..##.......          ..##.......
        //       ..##.###...    or    ..##.###...
        //       ..####.##..          ..####.##..
        //       ..##...##..          ..##...##..
        //       ..##....##.          ..##....##.
        //       ..##....##.          ..##....##.
        //       ..##....##.          ..##....##.
        //       ..##....##.          ..##....##.
        //       ..#....###.          ..#....###.
        //       #####.###.#          ###########
        //
        // Because of the '.' on the right of the bottom line (left
        // glyph) the above algorithm does not apply (the bottom line
        // is not a superset of the second-from-bottom).
        //
        // Already-melted serifs (and the above situation where the
        // 101->111 filter melts them) are expected to be very common.
        // They are tricky because heuristics for identifying
        // melted-serifs must avoid false-identifying
        //
        //      ##....##.            ##....##.
        //      ###...##.            ###...##.
        //      .#....##.            .#....##.
        //      .#....##.      or    .#....##.
        //      ##....##.            ##....##.
        //      .#....##.            .#....##.
        //      .##...##.            .##...##.
        //      .###..##.            .###..##.
        //      .#######.            .########
        //
        // as having serifs (if we did, we would end up removing
        // important feature information because removing the bottom
        // line from 'u' means it no longer has a top cup (an
        // identifying feature of u).
        //
        // If we can verify that the bottom line is a proper superset
        // of the second-bottom, and that it contains "on" pixels
        // further left and right than second-bottom, then we are
        // probably safe in removing it.

        bool result = false;
        if ( this->height() > 2 ) {
            u4 p1 = this->row(-1).pixels();
            int maxreg;
            BitOverlay::BitCount reg1 = BitOverlay::Regions(p1,&maxreg);

            if ( ( reg1 > 1 ) || (maxreg == 2) ) {
                // If there is only one region, it is dangerous
                // to apply this strategy because it can end up
                // deleting rows that shouldn't have been
                // deleted (see discussion above about 'o').
                // We only do serif-removal on 1-regions if
                // the region has length 2 (more extend to 3).
                
                u4 p2 = this->row(-2).pixels();
                u4 p3 = this->row(-3).pixels();
                
                BitOverlay::BitCount reg2 = BitOverlay::Regions(p2);
                BitOverlay::BitCount reg3 = BitOverlay::Regions(p3);
                
                //cerr << "reg1=" << reg1 << " reg2= " << reg2 << " reg3=" << reg3 << endl;
                
                if ( ( p1 != p2 ) && ( (p1 & p2) == p2 ) && ( reg1 == reg2 ) ) {
                    result = true;
                    //DEBUG: cerr << "REMOVING BOTBOM SERIFS" << endl;
                    if ( ( (p1 & p3) == p3 ) && ( reg1 == reg3 ) ) {
                        // We can use the p3 line!
                        this->row(-1).rawline(p3);
                        this->row(-2).rawline(p3);
                    } else {
                        this->row(-1).rawline(p2);
                    }
                }
            } else {
                // We have a 1-region bottom-line and potentially an
                // already-melted serif. We heuristically check if
                // bottom is a superset of second-bottom, and contains
                // set bits further left and right than second-bottom.
                // If so, it is assumed to be a melted serif and the
                // bottom line is replaced with second bottom.
                u4 p2 = this->row(-2).pixels();

                if ( ( p1 != p2 ) &&  // if they are the same, no use doing anything more (FUTURE FIX:  unless we are dealing with a large font in which multiple bottom lines have been melted!)
                     ( (p1 & p2) == p2 ) // bottom is superset of second-bottom
                    ) {
                    BitInfo bits1(p1);
                    BitInfo bits2(p2);
                    BitOverlay::BitCount reg2 = bits2.regionCount();

                    if ( (reg2 > 1 ) &&  // if second-bottom doesn't have multiple regions, it isn't a problem (FUTURE FIX: unless we are dealing with a large font in which multiple bottom lines have been melted!)
                        ( bits1.firstHigh() > bits2.firstHigh() ) &&
                         ( bits1.firstLow() < bits2. firstLow() ) ) {
                        this->row(-1).rawline(p2);
                        result = true;
                    }
                }

            }
        }

        if ( ( result ) && ( this->updated() ) ) {
            // If we were updated coming in, we maintain that by updating
            // again.  If we aren't updated, we don't update.
            this->updateMeta();
        }

        return result;
    }

    bool OptIm::removeTopSerifs() {
        // Top serifs are probably not as problematic as bottom
        // serirs, but the same algorithm can be applied to fix
        // up letters like 'u'.

        bool result = false;

        if ( this->height() > 2 ) {
            u4 p1 = this->row(0).pixels();
            int maxreg;
            BitOverlay::BitCount reg1 = BitOverlay::Regions(p1,&maxreg);
            if ( ( reg1 > 1 ) || (maxreg <= int(this->width()/2)) ) { 
                // For top serifs, we are allowing one-regions
                // as long as they are at most half the width.
                // This avoids 'o' tromps.
                
                u4 p2 = this->row(1).pixels();
                u4 p3 = this->row(2).pixels();
                
                BitOverlay::BitCount reg2 = BitOverlay::Regions(p2);
                BitOverlay::BitCount reg3 = BitOverlay::Regions(p3);
                
                //DEBUG: cerr << "reg1=" << reg1 << " reg2= " << reg2 << " reg3=" << reg3 << endl;
                
                if ( // ( p1 != p2 ) && 
                    ((p1 & p2) == p2 ) && 
                    ( reg1 == reg2 )
                    ) {
                    result = true;
                    //DEBUG: cerr << "REMOVING TOP SERIFS" << endl;
                    if ( ( (p1 & p3) == p3 ) && ( reg1 == reg3 ) ) {
                        // We can use the p3 line!
                        this->row(0).rawline(p3);
                        this->row(1).rawline(p3);
                    } else {
                        this->row(0).rawline(p2);
                    }
                    if ( this->updated() ) {
                        // If we are updated, we must reupdate.  If we aren't
                        // updated, we don't update.
                        this->updateMeta();
                    }
                    //DEBUG: this->print();
                }
            }
        }

        return result;
    }

    void OptIm::identifyFeatures(GlyphFeatures& features) {
        // Verify metadata
        this->updateMeta();

        // Data-structures
        const PixelMatrix& rows = this->rows();
        const PixelMatrix& cols = this->cols();

        GlyphFeatures::Variant variant;

        // First, perform vertical and horizontal line detection.
        u1 width  = this->width();
        u1 height = this->height();
        u1 xmax   = width  - 1;
        u1 ymax   = height - 1;
        u1 xhalf  = int(width/2)  - (1-(width%2));  // reduce by one if even
        u1 yhalf  = int(height/2) - (1-(height%2)); // reduce by one if even

        // Fudge factors

        // when exploring a location, range this far away from "target".
        //   - for top/bottom lines, dy establishes how from from borders
        //     to look (use the "most significant" line found).
        //   - for center lines, dcy/dcx establishes how from from 
        //     the center one/two lines to explore.
        int dy  = 1; 
        int dx  = 1;
        // If subtracting one from center causes us to overlap
        // with a column/row from left/top, don't subtract one!
        int dcy = (dy<(yhalf-1))? 1 : 0;
        int dcx = (dx<(xhalf-1))? 1 : 0;

        //DEBUG: cerr << "height=" << (int)height << " yhalf=" << (int)yhalf << " dcy=" << (int)dcy << endl;

        // Length determination:
        //  - Dominant lines have a length of 'width' (horizontal)
        //    or 'height' (vertical).
        //  - Recessive lines have a length greater equal 'wrec' (H)
        //    or 'hrec' (V).
        //  - Half lines have a length greater equal 'whalf' (H)
        //    or 'hhalf' (V).
        int wrec  = width - 1;
        // for small heights, only 1 pixel difference. For bigger
        // heights, more allowed. FUTURE FIX: Should be a fraction of
        // the height, not constant 1 or 2!!! The constants (or
        // fraction) are optimizable parameters!.
        int hrec  = height - 1; if (height >= 8) { --hrec; }
        // these differ from xmid and ymid in that we do no reduce by 1 if odd!
        int whalf = int(width/2); if ( whalf < 2 ) { whalf = 2; }
        int hhalf = int(height/2); if ( hhalf < 2 ) { hhalf = 2; }

        /// *********
        // Horizontal Lines (top, bottom, center)
        //  - if width is < 3, horizontal lines are meaningless
        //  - note that even many-pixel 'f's can be quickly reduced
        //    to width two depending on how much left/right cropping
        //    occurs - must be careful!
        if ( width > 2 ) {
            if ( height > 2 ) {
                // Top
                variant = this->analyzeLines(0,dy,rows,width,wrec,whalf);
                if ( variant != GlyphFeatures::Undef) {
                    features.addHorizontal(GlyphFeatures::Top, variant);
                }
                
                // Bottom
                variant = this->analyzeLines(ymax-dy,ymax,rows,width,wrec,whalf);
                if ( variant != GlyphFeatures::Undef) {
                    features.addHorizontal(GlyphFeatures::Bottom, variant);
                }
            }
            
            // Center
            variant = this->analyzeLines(yhalf-dcy,yhalf+dcy,rows,width,wrec,whalf);
            if ( variant != GlyphFeatures::Undef) {
                features.addHorizontal(GlyphFeatures::Center, variant);
            }
        }

        /// *********
        // Vertical Lines (left, right, center)
        //  - if height is <= 3, vertical lines are meaningless'

        if ( height > 3 ) {
            if ( width > 2 ) {
                // Left
                variant = this->analyzeLines(0,dx,cols,height,hrec,hhalf);
                if ( variant != GlyphFeatures::Undef) {
                    features.addVertical(GlyphFeatures::Left, variant);
                }
                
                // right
                variant = this->analyzeLines(xmax-dx,xmax,cols,height,hrec,hhalf);
                if ( variant != GlyphFeatures::Undef) {
                    features.addVertical(GlyphFeatures::Right, variant);
                }
            }

            // Center
            variant = this->analyzeLines(xhalf-dcx,xhalf+dcx,cols,height,hrec,hhalf);
            if ( variant != GlyphFeatures::Undef) {
                features.addVertical(GlyphFeatures::Center, variant);
            }
        }

        // ****************************************
        // Now perform holes/cups analysis
        //  - We first analyze the row black and hole distributions
        //     - Looking for a range of rows U-V such that U=b1, V=bk
        u2 blmax[4], blmin[4];
        u2 homax[4], homin[4];
	u1 blmaxi = 0, blmini = 0, blval = 0;
	u1 homaxi = 0, homini = 0, hoval = 0;
        bool blup = true;
	bool houp = true;
        for (int y = 0; y < height; ++y) {
            u1 black = rows[y].black();
            if ( blup ) {
                // We have been monotonically non-decreasing so far -
                // does this value continue that trend, or are we starting
                // down?
		if ( black < blval ) {
		    // We are going down!!  Last value (and index) marks a maximum.
		    cerr << "Found maximum black at y" << y-1 << "=" << (int)blval << endl;
		    blmax[blmaxi++] = (y-1)<<8 | blval;
		    blup = false;
		} else {
		    // We are continuing up
		}
            } else {
                // We have been monotonically non-increasing so far -
                // does this value continue that trend, or are we
                // starting up?
		if ( black > blval ) {
		    // We are going back up!  Last value (and index) marks a minimum
		    cerr << "Found minimum black at y" << y-1 << "=" << (int)blval << endl;
		    blmin[blmini++] = (y-1)<<8 | blval;
		    blup = true;
		} else {
		    // We are continuing down.
		}
            }
	    blval = black;

            u1 holes = rows[y].holes();
            if ( houp ) {
                // We have been monotonically non-decreasing so far -
                // does this value continue that trend, or are we starting
                // down?
		if ( holes < hoval ) {
		    // We are going down!!  Last value (and index) marks a maximum.
		    cerr << "Found maximum holes at y" << y-1 << "=" << (int)hoval << endl;
		    homax[homaxi++] = (y-1)<<8 | hoval;
		    houp = false;
		} else {
		    // We are continuing up
		}
            } else {
                // We have been monotonically non-increasing so far -
                // does this value continue that trend, or are we
                // starting up?
		if ( holes > hoval ) {
		    // We are going back up!  Last value (and index) marks a minimum
		    cerr << "Found minimum holes at y" << y-1 << "=" << (int)hoval << endl;
		    homin[homini++] = (y-1)<<8 | hoval;
		    houp = true;
		} else {
		    // We are continuing down.
		}
            }
	    hoval = holes;
        }
	if ( blup ) {
	    blmax[blmaxi++] = (height-1)<<8 | blval;
	    cerr << "Found maximum black at y" << height-1 << "=" << (int)blval << endl;
	} else {
	    blmin[blmini++] = (height-1)<<8 | blval;
	    cerr << "Found minimum black at y" << height-1 << "=" << (int)blval << endl;
	}
	if ( houp ) {
	    homax[homaxi++] = (height-1)<<8 | hoval;
	    cerr << "Found maximum holes at y" << height-1 << "=" << (int)hoval << endl;
	} else {
	    homin[homini++] = (height-1)<<8 | hoval;
	    cerr << "Found minimum holes at y" << height-1 << "=" << (int)hoval << endl;
	}
	// Analyze ranges
	for (int i = 0; i < blmaxi-1; ++i) {
	    //u1 s = blmax[i];
	    //u1 e = blmax[i+1];

	    // row 's' has a higher black count than either
	    // s-1 or s+1.  row 'e' has a higher black count
	    // than e-1 and e+1.  The rows between 's' and 'e'
	    // decrease from 's' and increase up to 'e'.


	    // 

	}
    }

    GlyphFeatures::Variant OptIm::analyzeLines(u1 min, u2 max, const PixelMatrix& matrix, u1 dom, u1 rec, u1 half) {
        // Service routine that iterates over one or more slices,
        // establishes whether the slice contains a "line" of black,
        // and if so, determines whether it is a dominant line
        // (maximum possible length = dom), recessive line (len >=
        // rec), half line (len >= half) or irrelevant (len < half).
        //
        // NOTE: This method is heuristic. It does not verify that
        // there is actually a contiguous sequence of black pixels
        // (anti-noise makes such an approach problematic). Instead,
        // it uses the cached 'black' and 'holes' values to establish
        // upper bounds on the length of a line if one does exist.
        //
        // The 'black' data specifies how many black pixels on in the
        // slice. If this value is 'dom' (maximum possible value, we
        // are certain it is Dominant and immediately stop). If the
        // value is >= 'rec', we assume it is Recessive but continue
        // with subsequent slices in case one of them is Dominant. If
        // the value is < 'rec', things become even more heuristic.
        // Often, 'half' is half of 'dom' (sometimes even reduced by
        // 1 from there).  Using a simple 'len >= half' heuristic can
        // easily end up reporting a HalfLine in situations where there
        // isn't anything remotely like a line
        //
        //                0000000         
        //              0 ######. 6 
        //              1 ##...## 4 
        //              1 ##....# 3 
        //              1 ##....# 3 
        //              1 ##....# 3 
        //              1 ##....# 3 
        //              1 ##....# 3 
        //                7711126   
        //          
        // 
        // Suppose the above OptIm is being assessed for a bottom (or
        // central) horizontal line. dom=7, rec=6, half=3. Since the
        // last row has 3 black pixels, this glyph would be identified
        // as having a Bottom/HalfLine horizontal using the naive
        // heuristic. However, we can make this a little more
        // intelligent by noting that the bottom (and center) rows all
        // have a hole. A row with 'b' black pixels and 'k' holes can
        // be heuristically downgraded to have only 'b-k' black pixels
        // (this takes into account the existence of at least k
        // breaks). Remember that 'b' is a naive estimator of actual
        // line length because it contains ALL black pixels, not just
        // those that are part of a conceptually contiguous line. If
        // the breaks represented by the 'k' holes are real, then the
        // line length is at the very least reduced by k. Of course,
        // it is possible these holes are due to anti-noise, in which
        // case they shouldn't have been removed from the black count.
        // However, since we are usually looking for the "best"
        // indicator of a line across multiple slices, such errors
        // will tend to be reduced.  We take the "best" line of the
        // set - if any of them reach HalfLine status even with the
        // conservative reduction based on holes, we are more
        // confident it really is a real half-line.
        //
        // Note that horizontal half-lines are of dubious utility
        // anyway, but the above will probably hold true for vertical
        // half-lines (which are much more common and useful).
        int debug = 0;

        GlyphFeatures::Variant variant = GlyphFeatures::Undef;
        DBG(1, "min=" << (int)min << " max=" << (int)max << " dom=" << (int)dom << " rec=" << (int)rec << " half=" << (int)half);
        for (int i = min; i <= max; ++i) {
            u1 len   = matrix[i].black();
            u1 holes = matrix[i].holes();
            DBG(1, "  i=" << i << " len=" << (int)len);
            if ( len >= dom ) {
                variant = GlyphFeatures::Dominant;
                DBG(1, "    Dominant");
                break;
            } else if ( len >= rec ) {
                variant = GlyphFeatures::Recessive;
                // don't break - next line might be dominant!
                DBG(1,"    Recessive");
            } else if ( len-holes >= half ) { 
                // see docs above for why we are subtracing holes from
                // len!
                if ( variant == GlyphFeatures::Undef ) {
                    // don't set if it is already "better"
                    variant = GlyphFeatures::HalfLine;
                    DBG(1,"    HalfLine");
                }
                // don't break - next line might be dominant!
            }
        }
        return variant;
    }

    void OptIm::test(int argc, const char* argv[]) {
        vector<unsigned int> rows;
        // #......    #......
        // #......    #......
        // #......    #......
        // #......    #......
        // #.###..    #.###..
        // ##...#.    ##...#.
        // #.....#    #.....#
        // #.....#    #..#..#
        // .#####.    .##..#.
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x5c);
        rows.push_back(0x62);
        rows.push_back(0x41);
        rows.push_back(0x41);
        rows.push_back(0x3e);
        
        OptIm im(rows, 7);
        cerr << im.str('#','.');
        
        cerr << "There are " << im.cblack(3) << " black pixels in column 3" << endl;
        cerr << "There are " << im.rholes(7) << " holes in row 7" << endl;

        im.updateMeta();
        cerr << im.str('#','.');        

        cerr << "There are " << im.cblack(3) << " black pixels in column 3" << endl;
        cerr << "There are " << im.rholes(7) << " holes in row 7" << endl;
        
    }
};

