using namespace std;
# include <iostream>
# include <fstream>
# include "WmhFormat.h"
# include "GocrModule.h"
# include "Env.h"
# include "Glyph.h" // GlyphLoopStart
# include "Line.h"

// C Dependencies - remove as they are replaced
# include "gocr.h"     // Job
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
# include "pgm2asc.h" // pgm2asc, getTextLine

extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);


// ####################################################################
// # Conjecture: An Extensible Optical Character Recognition Framework  #
// #                                                                  #
// # Copyright: Lesser GNU Public License (LGPL)                      #
// #                                                                  #
// # Overview:                                                        #
// #  - Conjecture is a collection of C++ classes, and an OCR engine    #
// #  - Conjecture is designed to allow customization at all levels     #
// #    and to encourage individuals to contribute incremental        #
// #    improvements in algorithms.                                   #
// #  - Overall design discussions can be found in                    #
// #      $SNROOT/docs/doxygen/html/index.html                        #
// #  - Conjecture is designed to interact with and build on other      #
// #    open-source OCR programs.                                     #
// ####################################################################

// Include

namespace Conjecture {

    WmhFormat::WmhFormat() {
        // Default Constructor
    }
    
    // Interface
    bool WmhFormat::execute(Page* page) {
        bool result = false;

        // Obtain the outfile
        const Env* env = page->env();
        const string& outfile = env->outfile();
        
        // Open the desired file.
        ofstream os(outfile.c_str());
        if ( !os ) {
            // FUTURE FIX: Report the reason for the failure!  How?
            cerr << "ERROR: Failed to open " << outfile << " for writing" << endl;
            result = false;
            
        } else {
            if ( env->verbose() <= 2 ) {
                // A chain around our necks!
		const GocrModule* module = dynamic_cast<const GocrModule*>(env->module());
		if ( module == NULL ) {
		    throw string("IncompatibleModuleException: WmhFormat requires GocrModule");
		}
                Job* job = module->job();
                
                // ******************************************************  
                store_boxtree_lines( job->cfg.mode );
                progress(90,0); /* progress is only estimated */
                
                // ******************************************************  
                // write out30.pgm
                if( job->cfg.verbose&32 ) debug_img("out30",job,2+4);
                
                progress(100,0); /* progress is only estimated */
                
                // ******************************************************  
                // Collesce characters into lines.
                int lc = 0;
                const char* line = getTextLine(lc++);
                int format = job->cfg.out_format;
                while (line) {
                    os << line;
                    if (format == HTML ) { os << "<br/>"; }
                    if ( format != XML ) { os << endl; }
                    line = getTextLine(lc++);
                }
                free_textlines();
            } else {
                // ***************************************************
                //     NEW CODE
                // ***************************************************
		cerr << "NOTE: Here in WmhFormat writing text" << endl;

                for (list<Element*>::iterator it = page->partsRef().begin(); it != page->partsRef().end(); ++it) {
                    Line* line = (*it)->asLine();
                    if ( line ) {
                        // Loop over all images in the line
			int i = 0;
                        GlyphLoopStart(line, glyph) {
			    ++i;
			    if ( ! glyph->invisible() ) {
				wchar_t c = glyph->unicode();
				if ( c == 0 ) { c = '_'; }
				os << (char)c;
			    }
                        } GlyphLoopEnd;
			os << endl;
                    } else {
                        cerr << "ERORR: Did not find a line" << endl;
                    }   
                }
            }
            
            // FUTURE FIX: If 'outfile' is '-', does closing it close
            // 'cout' in addition to 'os? If so, we should
            // special-case the close (don't close if 'outfile' is
            // '-'!
            os.close();
            
            result = true;
        }
        return result;
    }

};

