using namespace std;
# include <iostream>
# include <sstream> 
# include "OptIm15.h"
# include "Image.h"

// ####################################################################
// # Conjecture: An Extensible Optical Character Recognition Framework  #
// #                                                                  #
// # Copyright: GNU Public License                                    #
// #                                                                  #
// # Overview:                                                        #
// #  - Conjecture is a collection of C++ classes, and an OCR engine    #
// #  - Conjecture is designed to allow customization at all levels     #
// #    and to encourage individuals to contribute incremental        #
// #    improvements in algorithms.                                   #
// #  - Overall design discussions can be found in                    #
// #      $SNROOT/docs/doxygen/html/index.html                        #
// #  - Conjecture is designed to interact with and build on other      #
// #    open-source OCR programs.                                     #
// ####################################################################

// Old class - will probably be deleted. See 'OptIm' instead. Bruno
// was right - 15x15 is too limiting!

// Include

namespace Conjecture {
    OptIm15::OptIm15(const Image* image, const ImageArgs& args) {
        // We create an OptIm15 instance from the given Image. If the
        // size of the image exceeds 15 pixels in either dimension,
        // that dimension is truncated to 15 (top left corner of image
        // used).
        u2 szx = image->size().x;
        u2 szy = image->size().y;
        if ( szx > 15 ) { szx = 15; }
        if ( szy > 15 ) { szy = 15; }

        vector<unsigned int> rows;
        for (u2 y = 0; y < szy; ++y) {
            // We obtain the one-bit pixel values for this row of the image
            u2 pixels = 0;
            for (u2 x = 0; x < szx; ++x) {
                int val = args.apply(image,Coord(x,y));
                if ( val == ImageArgs::MinVal ) {
                    pixels |= (1<<(szx-1-x));
                }
            }
            rows.push_back(pixels);
        }
        init(rows,szx);
    }

    OptIm15::OptIm15(const vector<unsigned int>& inrows, unsigned char width) {
        this->init(inrows,width);
    }

    void OptIm15::init(const std::vector<unsigned int>& inrows, unsigned char width) {
        // Create rows of PixelLines from 'inrows'
        vector<PixelLine15>& rows = this->rowsRef();
        for (unsigned int i = 0; i < inrows.size(); ++i) {
            rows.push_back(PixelLine15(inrows[i]));
        }
        unsigned char height = rows.size();
        
        // Create columns of PixelsLines from 'rows'
        vector<PixelLine15>& cols = this->colsRef();
        for (int c = width-1; c >= 0; --c) {
            unsigned int p = 0;
            for (unsigned int r = 0; r < height; ++r) {
                if ( rows[r].get(c) ) { p |= (1<<r); }
            }
            cols.push_back(PixelLine15(p));
        }
        //DEBUG: cerr << this->rawstr(cols,'#','.',height) << endl;
        
        this->widthIs(width);
    }

    // Interface
    std::string OptIm15::str(char on, char off) const { // , unsigned char size) const {
        stringstream s;
        u2 size = this->width();
        
        // Obtain the columns
        const vector<PixelLine15>& cols = this->cols();
        
        // Print out an initial line representing column hole counts per
        // row. Since PixelLine15::str prints out row holes in 2 digits,
        // then space, then row data, then space, then black count in 2
        // digits, we use the same format for this header (and the footer
        // later).
        s << "  " << " ";
        for (vector<PixelLine15>::const_iterator it = cols.begin(); it != cols.end(); ++it) {
            unsigned char holes = it->holes();
            char h;
            if ( holes > 9 ) {
                h = char( (holes-9)-(int('a')) );
            } else {
                h = char( int('0')+holes);
            }
            s << h;
        }
        s << endl;
        
        // Now print the rows
        s << this->rawstr(this->rows(), on,off, size);
        
        // And now print a footer consisting of the column black counts
        s << "  " << " ";
        for (vector<PixelLine15>::const_iterator it = cols.begin(); it != cols.end(); ++it) {
            unsigned char black = it->black();
            char h;
            if ( black > 9 ) {
                h = char( (int('a')) + (black-10) );
            } else {
                h = char( int('0')+black);
            }
            s << h;
        }
        s << endl;
        
        // Return result
        return s.str();
    }

    std::string OptIm15::rawstr(const vector<PixelLine15>& vec, char on, char off, unsigned char size) const {
        stringstream s;
        for (vector<PixelLine15>::const_iterator it = vec.begin(); it != vec.end(); ++it) {
            s << it->str(on,off,size) << endl;
        }
        return s.str();
    }
    
    void OptIm15::test(int argc, const char* argv[]) {
        vector<unsigned int> rows;
        // #......    #......
        // #......    #......
        // #......    #......
        // #......    #......
        // #.###..    #.###..
        // ##...#.    ##...#.
        // #.....#    #.....#
        // #.....#    #..#..#
        // .#####.    .##..#.
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x40);
        rows.push_back(0x5c);
        rows.push_back(0x62);
        rows.push_back(0x41);
        rows.push_back(0x41);
        rows.push_back(0x3e);
        
        OptIm15 im(rows, 7);
        cerr << im.str('#','.');
        
        cerr << "There are " << im.cblack(3) << " black pixels in column 3" << endl;
        cerr << "There are " << im.rholes(7) << " holes in row 7" << endl;
        
    }

};

