# ifndef Conjecture_Coord_h
# define Conjecture_Coord_h Conjecture_Coord_h

// Insert SVN header substitution variable here
// Class Champion: Unassigned

# include "Root.h"
# include <stdint.h>  // uint8_t, uint16_t, etc.

namespace Conjecture {
    /**
     * \brief A simple 2-D Euclidean point.
     *
     * This class is used both for Image coordinates (x/y), and for
     * Image dimensions (width/height).
     *
     * The 'x' and 'y' fields are public.
     */
    class Coord : public Root {
      public:
        u2 x; // horizontal pixel position
        u2 y; // vertical pixel position

        // NOTE: Intentionally not defining a default constructor.
        // Forces us not to be careless about initializing the
        // values, since <0,0> is rarely the correct value.
        Coord(u2 x,u2 y) { this->x = x; this->y = y; }

        Coord(const Coord& coord) { this->x = coord.x; this->y = coord.y; }
        std::string str(int prec = 2) const; // printable representation
        
        Coord operator+(const Coord& c) const { return Coord(this->x+c.x,this->y+c.y); }
        Coord operator+(u2 val) const { return Coord(this->x+val,this->y+val); }
        
        Coord& operator+=(const Coord& c) { this->x += c.x; this->y += c.y; return *this; }
        Coord& operator+=(u2 val) { this->x += val; this->y += val; return *this; }
        
        Coord operator-(const Coord& c) const { return Coord(this->x-c.x,this->y-c.y); }
        Coord operator-(u2 val) const { return Coord(this->x-val,this->y-val); }
        
        Coord& operator-=(const Coord& c) { this->x -= c.x; this->y -= c.y; return *this; }
        Coord& operator-=(u2 val) { this->x -= val; this->y -= val; return *this; }
        
        /**
         * Returns true if receiver has smaller *area* than argument.
         * Is this the semantics we want?
         *
         */
        bool operator<(const Coord& c) const { return ( (this->x*this->x+this->y*this->y) < (c.x*c.x + c.y*c.y)) ? true : false; }
        
        /**
         * Returns true if this Coord is within the region formed by
         * 'topleft' and 'bottomright'.
         *
         */
        bool withinBounds(const Coord& topleft, const Coord& bottomright) const;
    };
};

# endif // Conjecture_Coord_h

