using namespace std;
# include <iostream>
# include <string>

// Unit testing support currently requires that we include all
// the class headers.  When we switch to a dlsym mechanism this
// won't be necessary.
# include "Env.h"
# include "Glyph.h"
# include "Image.h"
# include "Line.h"
# include "Page.h"
# include "Region.h"
# include "Root.h"
# include "Word.h"
# include "Element.h"
# include "OptIm.h"
# include "OptIm15.h"
# include "PixelLine.h"
# include "PixelLine15.h"
# include "CpuTimer.h"
# include "PixFilt.h"
# include "BitOverlay.h"
# include "BitInfo.h"
# include "StrManip.h"
# include "GlyphFeatures.h"

using namespace Conjecture;

int
main(int argc, const char **argv) {

    try {
	// First, check for a special case.  If the first arg is -T,
	// it is a request to invoke a unit-testing method instead of
	// performing normal execution.
	
	if ( string(argv[1]) == "-T" ) {
	    // FUTURE FIX: Use dlsym to obtain the appropriate method - much more flexible!
	    if ( argc > 2 ) {
		string test = argv[2];
		
		if ( test == "Image" ) {
		    Image::test(argc,argv);
		    //} else if ( test == "Element" ) {
		    // CONFLICT WITH GOCR CODE - FIX THIS!
		    // Element::test(argc,argv);
		} else if ( test == "Env" )  	    { Env::test(argc,argv); }
		else if ( test == "Page" )   	    { Page::test(argc,argv); }
		else if ( test == "Region" ) 	    { Region::test(argc,argv); }
		else if ( test == "Line" )   	    { Line::test(argc,argv); }
		else if ( test == "Word" )   	    { Word::test(argc,argv); }
		else if ( test == "Glyph" )  	    { Glyph::test(argc,argv); }
		else if ( test == "PixelLine15")    { PixelLine15::test(argc,argv); }
		else if ( test == "PixelLine" )     { PixelLine::test(argc,argv); }
		else if ( test == "OptIm15" )       { OptIm15::test(argc,argv); }
		else if ( test == "OptIm" )         { OptIm::test(argc,argv); }
		else if ( test == "CpuTimer" )      { CpuTimer::test(argc,argv); }
		else if ( test == "PixFilt" )       { PixFilt::test(argc,argv); }
		else if ( test == "BitOverlay" )    { BitOverlay::Test(argc,argv); }
		else if ( test == "BitInfo" )       { BitInfo::Test(argc,argv); }
		else if ( test == "StrManip" )      { StrManip::Test(argc,argv); }
		else if ( test == "GlyphFeatures" ) { GlyphFeatures::Test(argc,argv); }
		else {
		    cerr << "WARN: Unit testing of class " << test << " not yet supported" << endl;
		}
	    } else {
		cerr << "ERROR: -T requires an argument specifying the class to test" << endl;
	    }
	    
	} else {
	    
	    // We create an Env instance.
	    Env* env = new Env(argc, argv);
	    
	    // Obtain the current page (identified by command-line args)
	    Page* page = env->currentPage();
	    
	    // Process the page
	    page->process();
	    
	    // Now wrapup.
	    env->wrapup();
	}
      
    } catch(std::string& e) {
	// FUTURE FIX: Will start throwing more intelligent exceptions later
	cerr << endl << e << endl;
    
  }

  return(0);
}
