# ifndef Conjecture_BitInfo_h
# define Conjecture_BitInfo_h Conjecture_BitInfo_h

// Insert SVN header substitution variable here
// Class Champion: Unassigned

// Created 2006/06/10 by wade

// IMPORTANT: NO VIRTUAL METHODS!

# include "BitOverlay.h"  // parent

namespace Conjecture {
    
    /** *****************************************************************
     * \class BitInfo
     *
     * \brief Pre-computed bit-level information about an 32-bit word.
     *
     * This is an extension of BitOverlay that precomputes all
     * relevant information and caches it in fields. The constructor
     * performs a single pass over the bit-sequence to establish all
     * of the information of interest and stores that info in fields.
     *
     * In deciding when to use BitOverlay and when to use BitInfo,
     * almost any time more than one method from BitInfo is needed on
     * a particular bit sequence, using BitInfo is probably more
     * efficient.
     *
     * The only downside is that we have serious code redundancy - the
     * individual methods in BitOverlay perform individual tasks,
     * while the constructor in BitInfo performs those some tasks in
     * one monolithic loop (calling individual methods in the
     * constructor would defeat the entire purpose of having BitInfo -
     * it is supposed to be faster than calling individual methods).
     *
     *******************************************************************/

    class BitInfo : public BitOverlay {
      public:
        // Constructors/Destructors
        BitInfo(BitOverlay::Layout layout);
        
        // Accessors
        inline const BitOverlay::BitCount & largestRegion() const { return this->_largestRegion; }
        inline const unsigned short*        regions()       const { return this->_regions; }
        inline const BitOverlay::BitCount & regionCount()   const { return this->_regionCount; }
        inline const BitOverlay::BitPos &   firstLow()      const { return this->_firstLow; }
        inline const BitOverlay::BitPos &   firstHigh()     const { return this->_firstHigh; }
        
        // Input/Output
        std::ostream& print(std::ostream& os = std::cout, const std::string& indent = "") const;
        
        // Interface
        static void Test(int argc = 0, const char* argv[] = NULL);
        
      protected:
        // Accessors
        inline void            largestRegionIs(const BitOverlay::BitCount & largestRegion) { this->_largestRegion = largestRegion; }
        inline void            regionCountIs(const BitOverlay::BitCount & regionCount) { this->_regionCount = regionCount; }
        inline void            firstLowIs(const BitOverlay::BitPos & firstLow) { this->_firstLow = firstLow; }
        inline void            firstHighIs(const BitOverlay::BitPos & firstHigh) { this->_firstHigh = firstHigh; }
        
        // Methods 
        
      private: 
        // Accessors
        inline BitOverlay::BitCount &  largestRegionRef() { return this->_largestRegion; }
        inline unsigned short* regionsRef() { return this->_regions; }
        inline BitOverlay::BitCount &  regionCountRef() { return this->_regionCount; }
        inline BitOverlay::BitPos &    firstLowRef() { return this->_firstLow; }
        inline BitOverlay::BitPos &    firstHighRef() { return this->_firstHigh; }
        
        // Methods 
        
        // State
        BitOverlay::BitCount  _largestRegion;

        // could dynamically allocate smaller size, but this is faster.
        // Encodes both 'start' (least significant) and 'size' of region.
        // High 4-bits for start, 
        unsigned short        _regions[16]; 
        BitOverlay::BitCount  _regionCount;
        BitOverlay::BitPos    _firstLow;
        BitOverlay::BitPos    _firstHigh;
    };
}

# endif // Conjecture_BitInfo_h

