using namespace std;
# include <iostream>
# include "CpuTimer.h"

// Include

namespace Conjecture {

    CpuTimer::CpuTimer() {
	// Default Constructor
    }
    
    // Interface
    void CpuTimer::start() {
	// Start the timer running (cancel any timing currently occuring)
	this->getInfo(this->startRef());
	this->runningIs(true);
    }

    void CpuTimer::stop() {
	// Stop the timer so that multiple readings with the same
	// time can be performed.  If it is already stopped, don't
	// do anything.
	if ( this->running() ) {
	    this->getInfo(this->readingRef());
	    this->runningIs(false);
	}
    }

    void CpuTimer::print(ostream & os) const { 
	// Print out a full summary of the cpu time used since timer started
	//((Wmh_CpuTimer *)this)->stop();  // violating 'const' - should we bother stopping?
	os << "Timing results (in microseconds):" << endl;
	os << "   system : " << setw(10) << microtimeSystem() << endl;
	os << "   user   : " << setw(10) << microtimeUser() << endl;
	os << "   total  : " << setw(10) << microtime() << endl;
    }
    
    void CpuTimer::printSummary(ostream & os) const {
	//((Wmh_CpuTimer *)this)->stop();  // violating 'const' - should we bother stopping?
	os << time() << "(" << timeUser() << "u" << timeSystem() << "s)";
    }
    
    void CpuTimer::printMilliSummary(ostream & os) const {
	//((Wmh_CpuTimer *)this)->stop();  // violating 'const' - should we bother stopping?
	os << millitime() << "(" << millitimeUser() << "u" << millitimeSystem() << "s)";
    }
    
    void CpuTimer::printMicroSummary(ostream & os) const {
	//((Wmh_CpuTimer *)this)->stop();  // violating 'const' - should we bother stopping?
	os << microtime() << "(" << microtimeUser() << "u" << microtimeSystem() << "s)";
    }

    unsigned long CpuTimer::read() const {
	// Read the current timer value
	const struct rusage & start = this->start();
	struct rusage now = this->reading();  // is this copy intentional, or should it be by reference?
	if ( this->running() ) { this->getInfo(now); }
	
	return
	    ((now.ru_utime.tv_sec - start.ru_utime.tv_sec) * 1000000) +
	    (now.ru_utime.tv_usec - start.ru_utime.tv_usec) +
	    ((now.ru_stime.tv_sec - start.ru_stime.tv_sec) * 1000000) +
	    // purify is indicating that these fields are not being initialized?
	    (now.ru_stime.tv_usec - start.ru_stime.tv_usec);
    }
    
    unsigned long CpuTimer::readUser() const {
	// Read the current timer value (just user time)
	const struct rusage & start = this->start();
	struct rusage now = this->reading();
	if ( this->running() ) { this->getInfo(now); }
	
	return
	    ((now.ru_utime.tv_sec - start.ru_utime.tv_sec) * 1000000) +
	    (now.ru_utime.tv_usec - start.ru_utime.tv_usec);
    }
    
    unsigned long CpuTimer::readSystem() const {
	// Read the current timer value (just system time)
	const struct rusage & start = this->start();
	struct rusage now = this->reading();
	if ( this->running() ) { this->getInfo(now); }
	
	return
	    ((now.ru_stime.tv_sec - start.ru_stime.tv_sec) * 1000000) + 
	    (now.ru_stime.tv_usec - start.ru_stime.tv_usec);
    }

    void CpuTimer::test(int argc, const char* argv[]) {
	CpuTimer timer;
	for (int i = 0; i < 1000000; ++i) {
	    double j = i * 3.1415926535897932;
	    j += 1;
	}
	cout << "Total microsecond used: " << timer.millitime() << endl;
    }

};

