using namespace std;
# include <iostream>
# include "System.h"

# include <dirent.h>  // opendir

// ####################################################################
// # Conjecture: An Extensible Optical Character Recognition Framework  #
// #                                                                  #
// # Copyright: Lesser GNU Public License (LGPL)                      #
// #                                                                  #
// # Overview:                                                        #
// #  - Conjecture is a collection of C++ classes, and an OCR engine    #
// #  - Conjecture is designed to allow customization at all levels     #
// #    and to encourage individuals to contribute incremental        #
// #    improvements in algorithms.                                   #
// #  - Overall design discussions can be found in                    #
// #      $SNROOT/docs/doxygen/html/index.html                        #
// #  - Conjecture is designed to interact with and build on other      #
// #    open-source OCR programs.                                     #
// ####################################################################

// Include
# include <stdlib.h>

namespace Conjecture {

  System::System() {
    // Default Constructor
  }

  // Interface

    bool System::Mkdir(const string& path) {
	// FUTURE FIX: If we can find a platform-robust mechanism
	// that doesn't use system, it would be much more efficient!
	DIR* dir = opendir(path.c_str());
	bool result = false;
	if ( dir == NULL ) {
	    cerr << "NOTE: Creating directory " << path <<endl;
	    result = ShellCommand("mkdir -p " + path);
	} else {
	    result = true;
	}
	return result;
    }

    bool System::Mkpath(const string& path) {
	bool result;
	size_t i = path.find_last_of("/");
	if ( i == string::npos ) {
	    // No directory portion was found, which means
	    // we want to verify that the current directory
	    // exists, which is a noop - of course it does.
	    //
	    // This probably means the client is using
	    // this method incorrectly.
	    cerr << "WARN: No directory portion in " << path << " - did you mean to use Mkdir instead of Mkpath?" << endl;
	    result = true;
	} else {
	    // We have found the index of the last '/' - the dir to
	    // create is everything before that.
	    string dir = path.substr(0,i);
	    //DEBUG: cerr << "HERE MAKING " << dir << " for " << path << endl;
	    result = Mkdir(dir);
	}
	return result;
    }

    bool System::ShellCommand(const string& command) {
	bool result = false;
	int res = ::system(command.c_str());
	if ( res ) {
	    result = false;
	} else {
	    result = true;
	}
	return result;
    }

};

