package Wmh::Extensions;
use strict;

=head1 NAME

Wmh::Extensions -

=head1 SYNOPSIS

use Wmh::Extensions ();

=head1 DESCRIPTION


=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use IO::File;
use Wmh::Object;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(Wmh::Object);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw();


package main;

sub defineFunction {
    my ($package, $name, $defn) = @_;
    my $hashName = '\%' . $package . '::';
    my $hash = eval($hashName);

    if ( defined( $hash->{$name} ) ) {
          print "Function '$name' in package '$package' already defined.\n";
    } else {
          print "Function '$name' in package '$package' does not exist - adding a definition.\n";
          eval("package $package; $defn");
          if ( $@ ){
              die("Failed to evaluate '$name' - $@");
          }
    }
}


package IO::File;
use Carp;
sub new_safe {
    # Return an instance of IO::File opened on the given specification (as
    # accepted by the 'open' builtin.
    #
    # The signature has changed from the standard 'IO::File::new() and 'open()'
    # builtin.  Both of them allow for a mode and permissions - this one does not.
    # Instead, the arguments are a file specification (i.e. "<name" or ">name" or "com |"
    # etc.) and an optional error string. If the file cannot be opened, the program
    # will exit immediately with the provided error message (one will be automatically
    # generated if one is not provided).

    my ($type, $file, $opt_err) = @_;
    @_ >= 2 && @_ <= 3 or croak 'usage: new_safe $class $filespec [$error]';
    my $res = $type->new($file);

    if ( ! $res ) {
          # Based on the mode, generate an error message to print out
          my $err = $opt_err;
          my $arg2 = undef;   # hack for now - do we want to allow for MODE and PERMS?

          if ( ! defined($arg2) ) {
              # We are in the normal perl version in which both file and
              # mode are encoded in $file
              my $mode = $file;

              if ( $mode =~ m/^\s*(>{1,2})\s*(\S+)/ ) {
                  # Writing or appending
                  $err = sprintf("Failed to open '$2' for %s\n", ($1 eq '>') ?
                                 'writing' : 'appending');

              } elsif ( $mode =~ m/^\s*<?\s*(\S+)/ ) {
                  # Reading
                  $err = "Failed to open '$1' for reading";

              } elsif ( $mode =~ m/^\s*\|\s*(.*\S)/ ) {
                  # Piping out
                  $err = "Failed to open a pipe out to command '$1'";

              } elsif ( $mode =~ m/^\s*(.*)\|\s*$/ ) {
                  # Piping in
                  $err = "Failed to open a pipe in from command '$1'";

              }
          } else {
              # We are in the C version where $file is a file name and
              # $arg2 specifies how the file should be opened.
              my $mode = $arg2;

              $err = "Failed to open '$file' (mode '$mode' has not be englified yet)";
          }

          carp("ERROR: $err (\L$!\E)");
          exit(1);
    }

    return $res;
}

package CGI;

sub start {
    # Creates and returns a new CGI instance, but also prints out some
    # initial HTML text based on the %config arguments
    my ($class, %config) = @_;
    my $self = $class->new();  # we don't pass in an initializer.

    my $title = $config{'title'} || 'CGI Script written by Wade Holst (wade@cs.ualberta.ca)';
    print $self->header;
    print $self->start_html($title);
    print $self->h1($config{'header'}) if $config{'header'};
    $self->showDebugInfo() if $config{'debug'};

    return $self;
}

sub showDebugInfo {
    # Prints out the $self structure and a table of all environment
    # variables passed in.
    my ($self) = @_;

    print "\n<H1>DEBUG -> CGI instance structure</H1>\n";
    print "\n<PRE>\n" . Wmh::print_string($self) . "\n</PRE>\n";
    print "\n<H1>DEBUG -> Environment variable values</H1>\n";
    print '<TABLE BORDER=1>' . "\n";
    foreach (sort keys(%ENV)) {
          printf('   <TR> <TH> %s <TD> %s </TR>' . "\n",
                 $_, $ENV{$_});
    }
    print '</TABLE>' . "\n";
}

############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################



########################## Instance Methods ########################

###
# Accessors

# End Accessors
###

###
# User Code
#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: Wmh::Extensions::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('Wmh::Extensions'))) or carp(sprintf('Wmh::Extensions::initState: NOT $self->isa(Wmh::Extensions)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
    });
}}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4



=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         