package Wmh::Object;
use strict;

=head1 NAME

Wmh::Object -

=head1 SYNOPSIS

use Wmh::Object ();

=head1 DESCRIPTION


=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use IO::File;
use Wmh::Root;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(Wmh::Root);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw();



sub new {
    my ($class) = @_;
    my $self = {};

    bless $self, $class;

    # This code is crashing - why?
    $self->initState();   # usually will call subclasses 'initState'!

    return $self;
}

# Facility for extending superclass structure during creation

sub __EXTEND_LAYOUT {
    # Make this method less likely to conflict with user-defined versions!
    return &extend(@_);
}

sub extend {
    my ($self,$hash) = @_;
    my $key;
    foreach $key (keys(%{$hash})) {
        printf("WARNING: Key '$key' already exists in base - overwritten in %s\n",
               $self->myclass())
          if ( defined($self->{$key} ) );
        $self->{$key} = defined($hash->{$key}) ? $hash->{$key} : undef;
    }
}

# sub initState {
    # So that Pxx code works properly.
    # Need to fix this issue, since the generated Pxx code will not work when
    #   one subclasses on some class besides this Wmh::Object class - the code
    #   currently expects everyone to define initState()!
# }

sub myclass {my $res = ref($_[0]);  $res =~ s/^([^=]+)=(.*)/$1/; return $res; }
sub perlclass {my $res = ref($_[0]);  $res =~ s/^([^=]+)=(.*)/$1/; return $res; }

####################### Virtual Methods ############################

# All subclasses can use this to define their state.  Arguments are
#   [0] = $self  (shifted out)
#   [1] = field
#   [2] = optional value
sub _setget {
#    printf "I'm HERE with $_[1] (max arg %d)\n", $#_;
    return ( $#_ == 1 ) ?
        $_[0]->{$_[1]} :
       ($_[0]->{$_[1]} = $_[2]);
}

sub _set { return ($_[0]->{$_[1]} = $_[2]); }
sub _get { return $_[0]->{$_[1]}; }

sub _setgetbool {
    return ( $#_ == 1 ) ?
        ((defined($_[0]->{$_[1]}) && ($_[0]->{$_[1]})) ? 1 : 0) :
        ($_[0]->{$_[1]} = ($_[2]) ? 1 : 0);
}
sub _setbool { return ($_[0]->{$_[1]} = ($_[2]) ? 1 : 0); }
sub _getbool { return ((defined($_[0]->{$_[1]}) && ($_[0]->{$_[1]})) ? 1 : 0); }

sub _setgethash {
#    print "NUM is $#_, 0 is $_[0], 1 is $_[1], 2 is $_[2]\n";
    return ( $#_ == 2 ) ?
        $_[0]->{$_[1]}->{$_[2]} :
       ($_[0]->{$_[1]}->{$_[2]} = $_[3]);
}

sub _sethash { $_[0]->{$_[1]}->{$_[2]} = $_[3]; }
sub _gethash { return $_[0]->{$_[1]}->{$_[2]}; }

sub _gethash_initarr {
    # Return a hash value, initializing it to an empty array if it is
    # currently undefined.
    my $res = $_[0]->{$_[1]}->{$_[2]};
    if ( ! defined($res) ) {
        $res = [];
        $_[0]->{$_[1]}->{$_[2]} = $res;
    }
    return $res;
}

sub _gethash_inithash {
    # Return a hash value, initializing it to an empty hash if it is
    # currently undefined.
    my $res = $_[0]->{$_[1]}->{$_[2]};
    if ( ! defined($res) ) {
        $res = {};
        $_[0]->{$_[1]}->{$_[2]} = $res;
    }
    return $res;
}

sub _setgethasharr {
    return ( $#_ == 2 ) ?
        $_[0]->{$_[1]}->[$_[2]] :
       ($_[0]->{$_[1]}->[$_[2]] = $_[3]);
}

sub _setgethashhasharr {
    return ( $#_ == 3 ) ?
        $_[0]->{$_[1]}->{$_[2]}->[$_[3]] :
       ($_[0]->{$_[1]}->{$_[2]}->[$_[3]] = $_[4]);
}

sub _sethashhasharr { $_[0]->{$_[1]}->{$_[2]}->[$_[3]] = $_[4]; }
sub _gethashhasharr { return $_[0]->{$_[1]}->{$_[2]}->[$_[3]]; }

sub _gethashkeys {return keys(%{$_[0]->{$_[1]}});}
sub _gethashkeyssorted {return sort keys(%{$_[0]->{$_[1]}});}

sub _gethashhash_initarr {
    # Return a hash hash value, initializing it to an empty array if it is
    # currently undefined.
    my $res = $_[0]->{$_[1]}->{$_[2]}->{$_[3]};
    print "HERE\n";
    if ( ! defined($res) ) {
        printf "to $_[2] is %s\n", $_[0]->{$_[1]}->{$_[2]};
        $_[0]->{$_[1]}->{$_[2]} = {} if (!defined($_[0]->{$_[1]}->{$_[2]}));
        $res = [];
        $_[0]->{$_[1]}->{$_[2]}->{$_[3]} = $res;
        printf "to $_[3] is %s\n", $_[0]->{$_[1]}->{$_[2]}->{$_[3]};
    }
    return $res;
}

# Given an array of fields, determine if state contains a nested hash
# with the listed fields.  If so, return a reference to the resulting
# item, otherwise, return undef
sub _hashref {
    my $hash = shift;
    my $res = undef;
    my $f = 'self';

    # Each of the fields except the last one must result in a HASH value
    while ( $#_ > 0 ) {
        $f = shift;
#       print "HASH is $hash, new field is $f\n";
        if ( ( ! defined($hash = $hash->{$f}) ) ||
             ( ref($hash) ne 'HASH' ) ) {
            $hash = undef;
            last;
        }
    }
    # Last field can be anything, but it must exist
    $f = shift;
    return ( defined($hash) ) ?
        ( ( defined($hash = $hash->{$f}) ) ? \$hash : undef ) :
        undef;
}

# Given an array of fields, return a reference to the resulting
# item.  Create inner hashes as necessary if they don't already exist.  If
# an inner field is not a hash, returns undef
sub _hashrefinit {
    my $hash = shift;
    my $res = undef;
    my $f = 'self';

    while ( $#_ > 0 ) {
        $f = shift;
#       print "HASH is $hash, new field is $f\n";
        if ( defined( $hash->{$f} ) ) {
            $hash = $hash->{$f};
            if ( ref($hash) ne 'HASH' ) {
                $hash = undef;
            }
        } else {
            $hash->{$f} = {};
            $hash = $hash->{$f};
        }
    }
    # Last field can be anything, but it must exist
    $f = shift;
    $hash->{$f} = undef if ! defined($hash->{$f});
    return \$hash->{$f};

}


#########:
# Purpose: Return a string representing my internal structure
# Args   : $self : ANYTHING, %format : list
# Notes  :
#   This function will work even when the first argument is not a blessed
#   reference.  Of course, an OO calling scheme won't be possible if the
#   item to be printed isn't blessed, but you can still use the good old
#   indirect-object calling scheme.
#
#   Examples:
#      $a = new ADND;               # assuming that ADND is a subclass of Wmh
#      print $a->print-string();    # OO calling scheme for blessed objects
#
# Return : text

sub print_string
{
    my $self = shift;
    return( &Wmh::print_string($self,@_) );
}

#########:
# Purpose: Print myself on the given filehandle (STDOUT by default)
# Args   :
# Notes  :
# Return :

# This can be redefined in subclasses to print out a much nicer versions of
# oneself
sub print {
    my $self = shift;
    return( &Wmh::print($self, @_) );
}

# This should not be redefined in subclasses
sub dump {
    my $self = shift;
    return( &Wmh::print($self, @_) );
}

sub subclassResponsibility {
    # Given a method name and a string representing the arguments (separated by
    # commas, with optional '=value' default value indicators and optional
    # 'opt_' prefix (to indicated optional argument), this method generates
    # a 'subclassResponsibility' error message so that the user knows to
    # add an appropriately defined method in whatever subclass they are
    # using.
    #
    # EXAMPLE:
    #   $self->subclassResponsibility('someMethod',
    #                                 '$arg1, $arg2 = "default"');

    my ($self, $method, $args) = @_;
    printf(STDERR "ERROR: Method '$method' must be implemented in subclass '%s'.\n",
           $self->myclass());
    my $i = 1;
    printf(STDERR "   Arguments to $method are:\n       [0] \$self\n");
    foreach (split(/\s*,\s*/, $args)) {
        m/^([^=]+)(?:=\s*(.*\S))/;
        printf(STDERR "       [%d] %-20s %s\n",
               $i, $1, (defined($2) ? ('(optional: default = \'' . $2 . '\')') : ''));
        ++$i;
    }
    exit(1);
}

sub typeCheck {
    # Verify that I am a blessed reference whose class matches $class.
    # For now, subtype checking is not implemented - the class of the
    # object must much exactly against the specifed class.

    my ($self, $class) = @_;
    if ( $self->myclass() eq $class ) {
        &croak("ERROR: Type checking %s against class '$class' failed.\n", $self);
    }
}

############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################



########################## Instance Methods ########################

###
# Accessors

# End Accessors
###

###
# User Code
#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: Wmh::Object::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('Wmh::Object'))) or carp(sprintf('Wmh::Object::initState: NOT $self->isa(Wmh::Object)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
    });
}}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4



=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         
