package XFig::Compound;
use strict;

=head1 NAME

XFig::Compound -

=head1 SYNOPSIS

use XFig::Compound ();

=head1 DESCRIPTION


=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use XFig::Polyline;
use IO::File;
use XFig::Box;
use XFig::Object;
use XFig::Text;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(XFig::Object);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw();




############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################

#line 1787 "fig.pxx"
sub newBoxedText {
    carp('Usage: XFig::Compound::newBoxedText(class, data, options)') unless (@_ >= 2);
    my($class) = $_[0]; CORE::ref($class) eq '' or carp(sprintf('XFig::Compound::newBoxedText: ref($class) = %s instead of .', CORE::ref($class))); 
    my($data) = $_[1]; ((!$data) || (CORE::ref($data) eq 'ARRAY')) or carp(sprintf('XFig::Compound::newBoxedText: ref($data) = %s instead of ARRAY.', ref($data))); 
    my(%options) = @_[2..$#_]; 
#line 1841 "fig.pxx"

    my $uml   = $options{uml}   || '';
    my $depth = $options{depth} || 100;

    # Initially, we create the compound object representing
    # the class box with top-left corner at <0,0>.  It can then
    # be moved elsewhere.

    # Do first pass over $data adding 'height' and 'width' fields
    # to each hash, and identifying the y values of the boxes.
    my $Width  = 0; # maximum width of all items
    my $Height = 0; # current height (y position)
    my @Bottoms; # the y values of the bottoms of each box
    my @text;
    my %defaults = (
                    margin    => 0,
                    align     => 'left',
                    bcolor    => $options{'bgcolor'} || 'Default',
                    color     => $options{'color'}   || 'Default',
                    depth     => $options{depth}     || $XFig::XFig::Defaults{depth},
                    size      => $XFig::XFig::Defaults{font_size},
                    thickness => $XFig::XFig::Defaults{thickness},
                   );

    my @objects;

    foreach my $i (0..$#$data) {
        my $box = $data->[$i];

        # The collection of text lines to render within this box
        my $lines    = $box->{lines} || die("ERROR: Missing 'lines' key in box $i\n");
        # The box-related config options
        my $boxconf  = $box->{box} || {};
        # The text-related config options
        my $textconf = $box->{text} || {};

        &fixOptions($boxconf);
        &fixOptions($textconf);

        #Wmh::print($box);Wmh::print($textconf);Wmh::print($boxconf); exit;

        ###
        # There are two kinds of margin:
        #   - the margin within the box surrounding all text lines (in $boxconf)
        #   - the margin around individual text lines (within the box margins), in individual lines (or $textconf for defaults)
        #
        # There are four margin values:
        #   - leftmargin
        #   - rightmargin
        #   - topmargin
        #   - bottommargin
        #
        # Assuming the left side of the box is at x=0, the start of
        # text is at the boxes left margin PLUS the line's left margin.
        # The end of text is at the boxes right side minus the boxes
        # right margin MINUS the lines' right margin.

        # This represents the width of the box margins, NOT including
        # the line margins.
        my $boxMarginWidth = $boxconf->{leftmargin} + $boxconf->{rightmargin};

        # We advance the vertical height by the boxes topmargin
        $Height += $boxconf->{topmargin};

        foreach my $line (@$lines) {
            # Compute the margins for this text widget (almost
            # always 0 for all margins).  Note that we do NOT
            # inherit the margin values from the $boxconf hash,
            # but DO inherit from $textconf.
            &fixOptions($line, $textconf);

            # Set some defaults (NOTE: These should be unnecessary now that we are using 'fixOptions' above!)
            $line->{align} ||= $textconf->{align};
            $line->{color} ||= $textconf->{color};
            $line->{size}  ||= $textconf->{size};
            $line->{depth} ||= $textconf->{depth};

            # Compute the width, height and vertical spacing of this text string.
            my $text = $line->{text} || die("ERROR: Missing text field within a line in box $i\n");
            my ($w,$h,$v) = &XFig::Text::textInfo($line->{size}, $text);
            $line->{width}    = $w;
            $line->{height}   = $h;
            $line->{vertical} = $v;

            # $Width represents the maximum width of all text strings
            # including box left/right margins and text left/right margins.
            my $xmargins = $boxMarginWidth + $line->{leftmargin} + $line->{rightmargin};
            my $width = $w + $xmargins;
            if ( $Width < $width ) { $Width = $width; }

            # $Height represents the current $y position (the baseline
            # of each text string, the bounds of each box).  We advance
            # it by the vertical spacing of the text string, as well
            # as the top margin for the text string

            # No longer using $v - leaves too much space.  $h is
            # better (20040512)
            #$Height += $v + $line->{topmargin};
            $Height += $h + $line->{topmargin};

            # We store the bottom-left corner of the text string, assuming
            # the left-side of the box is at x=0.  The string starts at
            # the boxes leftmargin plus the line left margin.
            # This position needs to be adjusted once we know the
            # maximum $Width to respect the 'align' request.
            $line->{'x'} = $boxconf->{leftmargin} + $line->{leftmargin};
            $line->{'y'} = $Height;

            # Now add in the bottom margin for this string
            $Height += $line->{bottommargin};

            #Wmh::print($line); exit;
        }

        # We advance the vertical height by bottomMargin and record it
        $Height += $boxconf->{bottommargin};

        push(@Bottoms, $Height);
    }

    ###
    # At this point, we have a maximum $Width and thus have all the
    # information necessary to create the text and box objects
    # represented by $data.
    my %sub_type = (
                    left   => 0,
                    center => 1,
                    right  => 2,
                    );
    my $y = 0;
    foreach my $i (0..$#$data) {
        my $box = $data->[$i];
        my $lines = $box->{lines};
        my $boxconf = $box->{box};

        ###
        # We first create the bounding box for all text in this
        # box.  Note that the following keys from $boxconf apply:
        #   thickness    --> thickness of border
        #   bcolor       --> pen_color (for border)
        #   bpattern     --> pattern for border (line_style)
        #
        #   depth        --> depth of the box
        #   color        --> fill_color
        #   pattern      --> area_fill
        my $depthadj = $boxconf->{depth} || 0;

        my $bottomy = $Bottoms[$i]-1;
        my $boxobj = XFig::Box->new( {
                                      bounds    => [0,$y,$Width,$bottomy],
                                      depth     => $depth + $depthadj,
                                      thickness => ($boxconf->{thickness} || $defaults{thickness} ),
                                   }
                                 );
        $boxobj->area_fill( $boxconf->{pattern} ) if $boxconf->{pattern};
        $boxobj->setFillColor( $boxconf->{'color'} ) if $boxconf->{'color'};
        $boxobj->setPenColor( $boxconf->{'bcolor'} ) if $boxconf->{'bcolor'};
        $boxobj->subcomment($box->{comment}) if $box->{comment};
        push(@objects, $boxobj);
        $y = $bottomy;

        ###
        # Now we create each individual Text object
        foreach my $line (@$lines) {
            my $x = $line->{'x'};
            my $y = $line->{'y'};
            my $align = $line->{align};
            # The alignment of text determines the <x,y> values that
            # XFig stores for the Text object.  For left alignment,
            # <x,y> is baseline left corner, for right alignment it
            # is baseline right corner, and for center alignment it
            # is baseline center.
            #
            # We know that $x is currently set to the left
            # baseline corner of the text.  For 'center' and
            # 'right' alignment we need to adjust this value.
            # Note that the 'sub_type' component of Xfig Text
            # objects is marked as left, center or right, and
            # that the x position of the object is interpreted
            # differently depending on this sub-type.  For
            # 'center', the x position is supposed to represent
            # the center of the text (so we set the x position
            # to be the middle point of the box itself).  For
            # 'right', the x position is supposed to be the
            # right-most x position (so we set it to the sum of
            # the box right margin and line right margin).
            # Remember that $Width reprsents the box width.
            if ( $align eq 'center' ) {
                $x = int($Width/2);
            } elsif ( $align eq 'right' ) {
                $x = $Width - $boxconf->{rightmargin} - $line->{rightmargin};
            }
            # else $align is 'left', and $x is correct as-is.

            my $textobj = XFig::Text->new( {
                                            text        => $line->{text},
                                            'sub_type'  => $sub_type{$align},
                                            'font_size' => $line->{size},
                                            'x'         => $x,
                                            'y'         => $y,
                                            depth       => $depth,
                                           }
                                  );
            $textobj->setPenColor( $line->{color} ) if $line->{color};
            push(@objects, $textobj);
        }
    }

    ####
    # Create the compound object
    my $self = $class->new({ 'ulx' => 0, 'uly' => 0,
                             'lrx' => $Width, 'lry' => $Bottoms[-1],
                             objects => [],
                           }
                          );
    if ( $uml ) {
        $self->comment("$uml: Compound object representing class diagram box");
    }
    foreach my $obj (@objects) {
        $self->addObject($obj);
    }

    return $self;
}

#line 1754 "fig.pxx"
sub new {
    carp('Usage: XFig::Compound::new(class, args)') unless (@_ == 2);
    my($class) = $_[0]; CORE::ref($class) eq '' or carp(sprintf('XFig::Compound::new: ref($class) = %s instead of .', CORE::ref($class))); 
    my($args) = $_[1]; ((!$args) || (CORE::ref($args) eq 'HASH')) or carp(sprintf('XFig::Compound::new: ref($args) = %s instead of HASH.', ref($args))); 
#line 1755 "fig.pxx"
    $args->{object_code} = 6;

    # Compound does not have any 'friendly' keys, so we
    # optimize and avoid the call to adjustArgs.
    my $self = $class->SUPER::new($args);

    # The only arg that is strictly necessary is 'objects' - we
    # can compute the bounding box by iterating over each object.
    if ( ! defined($args->{ulx}) ) {
        my $objects = $args->{objects};
        if ( $objects ) {
            my ($ulx,$uly,$lrx,$lry) = (1000000,1000000,-1000000,-1000000);
            foreach my $object (@$objects) {
                my ($minx,$miny,$maxx,$maxy) = $object->bbox();
                $ulx = $minx if $minx < $ulx;
                $uly = $miny if $miny < $uly;
                $lrx = $maxx if $maxx > $lrx;
                $lry = $maxy if $maxy > $lry;
            }
            $args->{ulx} = $ulx;
            $args->{uly} = $uly;
            $args->{lrx} = $lrx;
            $args->{lry} = $lry;
        } else {
            die("ERROR: Must specify at least the 'objects' arg for Compound objects\n");
        }
    }

    $self->initFields($args, qw(ulx uly lrx lry objects));
    return $self;
}

########################## Instance Methods ########################

###
# Accessors
#line 1750 "fig.pxx"
sub addObject            {  push(@{$_[0]->{objects}}, $_[1]);  }
#line 1747 "fig.pxx"
sub incrLrx              {  return $_[0]->{lrx} += (defined($_[1]) ? $_[1] : 1); }
#line 1748 "fig.pxx"
sub incrLry              {  return $_[0]->{lry} += (defined($_[1]) ? $_[1] : 1); }
#line 1745 "fig.pxx"
sub incrUlx              {  return $_[0]->{ulx} += (defined($_[1]) ? $_[1] : 1); }
#line 1746 "fig.pxx"
sub incrUly              {  return $_[0]->{uly} += (defined($_[1]) ? $_[1] : 1); }
#line 1747 "fig.pxx"
sub lrx                  {  return ( $#_ == 0 ) ? $_[0]->{lrx} : ( $_[0]->{lrx} = $_[1] );  }
#line 1748 "fig.pxx"
sub lry                  {  return ( $#_ == 0 ) ? $_[0]->{lry} : ( $_[0]->{lry} = $_[1] );  }
#line 1750 "fig.pxx"
sub numObjects           {  return scalar(@{$_[0]->{objects}});  }
#line 1750 "fig.pxx"
sub object               {  if ( @_ == 2 ) { return $_[0]->{objects}->[$_[1]]; } else { $_[0]->{objects}->[$_[1]] = $_[2]; return $_[2]; }  }
#line 1750 "fig.pxx"
sub objects              {  if ( $#_ == 0 ) { return (wantarray) ? @{$_[0]->{objects}} : $_[0]->{objects}} else { $_[0]->{objects} = $_[1]; return (wantarray) ? @{$_[1]} : $_[1] }  }
#line 1745 "fig.pxx"
sub ulx                  {  return ( $#_ == 0 ) ? $_[0]->{ulx} : ( $_[0]->{ulx} = $_[1] );  }
#line 1746 "fig.pxx"
sub uly                  {  return ( $#_ == 0 ) ? $_[0]->{uly} : ( $_[0]->{uly} = $_[1] );  }
# End Accessors
###

###
# User Code
#line 2131 "fig.pxx"
sub adjustDepth {
    carp('Usage: XFig::Compound::adjustDepth(self, adj)') unless (@_ == 2);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Compound'))) or carp(sprintf('XFig::Compound::adjustDepth: NOT $self->isa(XFig::Compound)')); 
    my($adj) = $_[1]; CORE::ref($adj) eq '' or carp(sprintf('XFig::Compound::adjustDepth: ref($adj) = %s instead of .', CORE::ref($adj))); $adj =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Compound::adjustDepth: Variable $adj = %s does not match type int.', $adj)); 
#line 2134 "fig.pxx"
    foreach my $obj ($self->objects()) {
        $obj->depth( $obj->depth() + $adj );
    }
}

#line 2126 "fig.pxx"
sub bbox {
    carp('Usage: XFig::Compound::bbox(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Compound'))) or carp(sprintf('XFig::Compound::bbox: NOT $self->isa(XFig::Compound)')); 
#line 2128 "fig.pxx"
    return ($self->ulx(), $self->uly(), $self->lrx(), $self->lry());
}

#line 2101 "fig.pxx"
sub clone {
    carp('Usage: XFig::Compound::clone(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Compound'))) or carp(sprintf('XFig::Compound::clone: NOT $self->isa(XFig::Compound)')); 
#line 2102 "fig.pxx"
    my $result = { %$self };
    bless $result, $self->myclass();

    # We need to deep-copy 'objects'
    my @objects;
    foreach my $obj ($self->objects()) {
        push(@objects, $obj->clone());
    }
    $result->{objects} = \@objects;

    return $result;
}

#line 2068 "fig.pxx"
sub fixOptions {
    carp('Usage: XFig::Compound::fixOptions(hash, parent)') unless ( (@_ >= 1) && (@_ <= 2) );
    my($hash) = $_[0]; ((!$hash) || (CORE::ref($hash) eq 'HASH')) or carp(sprintf('XFig::Compound::fixOptions: ref($hash) = %s instead of HASH.', ref($hash))); 
    my($parent) = $_[1]; $parent = {} unless defined($parent); ((!$parent) || (CORE::ref($parent) eq 'HASH')) or carp(sprintf('XFig::Compound::fixOptions: ref($parent) = %s instead of HASH.', ref($parent))); 
#line 2073 "fig.pxx"
    foreach my $p (qw(left right top bottom)) {
        my $k = $p . 'margin';
        my $v = $hash->{$k};
        if ( ! defined($v) ) {
            my $axis = ( $p =~ m/^[lr]/ ) ? 'x' : 'y';
            $v = $hash->{"${axis}margin"};
            if ( ! defined($v) ) {
                $v = $hash->{margin};
                if ( ! defined($v) ) {
                    $v = $parent->{$k}; # this assumes that $parent has already been expanded so that left/right/top/bottom margins exist.
                    if ( ! defined($v) ) {
                        $v = 0;
                    }
                }
            }
            $hash->{$k} = $v;
        }
    }

    ###
    # This maps symbolic values to XFig values:
    foreach my $c (qw(color bcolor)) {
        if ( $hash->{$c} ) {
            $hash->{$c} = &XFig::XFig::colorIndexFor($hash->{$c});
        }
    }
}

#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: XFig::Compound::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Compound'))) or carp(sprintf('XFig::Compound::initState: NOT $self->isa(XFig::Compound)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
        objects => [],
        # uly => '',
        # ulx => '',
        # lry => '',
        # lrx => '',
    });
}}

#line 2115 "fig.pxx"
sub rawTranslate {
    carp('Usage: XFig::Compound::rawTranslate(self, dx, dy)') unless (@_ == 3);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Compound'))) or carp(sprintf('XFig::Compound::rawTranslate: NOT $self->isa(XFig::Compound)')); 
    my($dx) = $_[1]; CORE::ref($dx) eq '' or carp(sprintf('XFig::Compound::rawTranslate: ref($dx) = %s instead of .', CORE::ref($dx))); $dx =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Compound::rawTranslate: Variable $dx = %s does not match type int.', $dx)); 
    my($dy) = $_[2]; CORE::ref($dy) eq '' or carp(sprintf('XFig::Compound::rawTranslate: ref($dy) = %s instead of .', CORE::ref($dy))); $dy =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Compound::rawTranslate: Variable $dy = %s does not match type int.', $dy)); 
#line 2117 "fig.pxx"
    $self->ulx( $self->ulx() + $dx );
    $self->lrx( $self->lrx() + $dx );
    $self->uly( $self->uly() + $dy );
    $self->lry( $self->lry() + $dy );
    foreach my $obj ($self->objects()) {
        $obj->rawTranslate($dx, $dy);
    }
}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4

=head1 newBoxedText(Array $data, ... %options)
##
This method creates a box containing lines of text.
Actually, multiple boxes can be created, each with its own
configuration information.

Each element of $data is a hash containing the following:

comment            # optional: used when writing .fig files to make output more readable

box                # optional: hash containing zero or more of these:
thickness      #    thickness of border box
bcolor         #    color of border
color          #    color of box (implies fill pattern 20 unless actual boxpattern specified).
depth          #    depth of box (not text)
bpattern       #    border pattern
pattern        #    fill pattern
leftmargin     #    amount of space between left side of box and start of text
rightmargin    #    amount of space between right side of box and end of text
topmargin      #    amount of space between top side of box and top of first line of text
bottommargin   #    amount of space between bottom side of box and baseline of last line of text

text               # optional: hash containing zero or more of these:
color          #    default text color for all lines in this "chunk"
align          #    default alignment used by lines of text in this box
size           #    default font size used by lines of text in this box
depth          #    the depth of this box and its text.
leftmargin     #    optional - space between left wall of box and start of text
rightmargin    #    optional - space between right wall of box and end of text
topmaargin     #    optional - space between baseline of previous line and top of current line
bottommargin   #    optional - space between basline of current line and top of next line

lines              # required: an array of hashes containing the following keys:
text           #    required - the text string (verbatim)
size           #    optional - text size (default used if not given)
align          #    optinoal - alignment: 'left', 'center' or 'right'
color          #    optional - text color: color name (see %Colors hash).  Uses parent 'color' or a default by default
leftmargin     #    optional - space between left wall of box and start of text
rightmargin    #    optional - space between right wall of box and end of text
topmargin      #    optional - space between baseline of previous line and top of current line
bottommargin   #    optional - space between basline of current line and top of next line

%options are:
uml   --> has two purposes.  First, it identifies the boxed text
as a class diagram box.  Second, it provides the class name,
This name will be used when generating object
comments (which in turn are inserted into the .fig
file as comments to make the file more readable).
If no 'uml' key is given, no comments are added.
depth  --> the default depth at which the compound object should be
rendered.  The various components of the compound can provide
'depth' config options which represent RELATIVE adjustments
to this depth value.
bgcolor--> default background color to use.

=cut

=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 adjustDepth(int $adj)
This method adjusts the depth of each object in the compound
by $adj.

=cut

=head2 bbox()
Return the bounding box for this object

=cut

=head2 fixOptions(Hash $hash, Hash $parent = {})
##
This fixes margin options (if 'leftmargin' isn't given, look
for 'xmargin', then 'margin', then 'leftmargin' in $parent,
then 0.

=cut

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head2 rawTranslate(int $dx, int $dy)
Need to adjust the bounding coords and each object.

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         