package XFig::Root;
use strict;

=head1 NAME

XFig::Root -

=head1 SYNOPSIS

use XFig::Root ();

=head1 DESCRIPTION
Abstract superclass of all XFig classes

=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use IO::File;
use Wmh::Object;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(Wmh::Object);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw();




############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################

#line 33 "fig.pxx"
sub new {
    carp('Usage: XFig::Root::new(class)') unless (@_ == 1);
    my($class) = $_[0]; CORE::ref($class) eq '' or carp(sprintf('XFig::Root::new: ref($class) = %s instead of .', CORE::ref($class))); 
#line 34 "fig.pxx"
    my $self = $class->SUPER::new();
    return $self;
}

########################## Instance Methods ########################

###
# Accessors

# End Accessors
###

###
# User Code
#line 40 "fig.pxx"
sub initFields {
    carp('Usage: XFig::Root::initFields(self, args, fields)') unless (@_ >= 2);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Root'))) or carp(sprintf('XFig::Root::initFields: NOT $self->isa(XFig::Root)')); 
    my($args) = $_[1]; ((!$args) || (CORE::ref($args) eq 'HASH')) or carp(sprintf('XFig::Root::initFields: ref($args) = %s instead of HASH.', ref($args))); 
    my(@fields) = @_[2..$#_]; 
#line 49 "fig.pxx"
    foreach my $k (@fields) {
        # Violating accessor API here
        my $val = delete $args->{$k};
        if ( ! defined($val) ) { 
            $val = $XFig::XFig::Defaults{$k};
            if ( ! defined($val) ) {
                croak("Missing required '$k' parameter");
            }
        }
        #printf(STDERR "NOTE:    Setting field %-15s = %s\n", $k, $val);
        $self->{$k} = $val;
    }
}

#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: XFig::Root::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Root'))) or carp(sprintf('XFig::Root::initState: NOT $self->isa(XFig::Root)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
    });
}}

#line 63 "fig.pxx"
sub nextline {
    carp('Usage: XFig::Root::nextline(fp)') unless (@_ == 1);
    my($fp) = $_[0]; 
#line 67 "fig.pxx"

    # Grab the next line
    my $res = <$fp>;
    return undef unless defined($res);

    # Keep grabbing lines until we get a non-comment
    while ( $res =~ m/^\#/ ) {
        $res = <$fp>;
    }

    # Strip leading and trailing whitespace
    $res =~ s/^\s*//;
    $res =~ s/\s*$//;

    return $res;
}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4



=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 initFields(Hash $args, ... @fields)
###
This method is used by subclasses to "safely" assign fields.
- the user provides a hash of arguments to a constructor.
- each constructor invokes 'initFields', passing in the $args
hash and an array of fields, this method sets the specified
field values from values in $args, then deletes the key/value
from $args.
- this allows us to detect arguments that are not recognized.

=cut

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head2 nextline(any $fp)
All classes may need to obtain a xfig data line from a
file-handle.  This method does so, and is available to all
XFig classes.

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         
