package XFig::ShapeObject;
use strict;

=head1 NAME

XFig::ShapeObject -

=head1 SYNOPSIS

use XFig::ShapeObject ();

=head1 DESCRIPTION
Abstract superclass of
=head1     ARC
=head1     ELLIPSE
=head1     POLYLINE
=head1     SPLINE

=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use IO::File;
use XFig::BasicObject;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(XFig::BasicObject);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw();




############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################

#line 2413 "fig.pxx"
sub new {
    carp('Usage: XFig::ShapeObject::new(class, args)') unless (@_ == 2);
    my($class) = $_[0]; CORE::ref($class) eq '' or carp(sprintf('XFig::ShapeObject::new: ref($class) = %s instead of .', CORE::ref($class))); 
    my($args) = $_[1]; ((!$args) || (CORE::ref($args) eq 'HASH')) or carp(sprintf('XFig::ShapeObject::new: ref($args) = %s instead of HASH.', ref($args))); 
#line 2415 "fig.pxx"
    my $self = $class->SUPER::new($args);
    $self->initFields($args, qw(line_style thickness fill_color area_fill style_val));
    return $self;
}

########################## Instance Methods ########################

###
# Accessors
#line 2408 "fig.pxx"
sub area_fill            {  return ( $#_ == 0 ) ? $_[0]->{area_fill} : ( $_[0]->{area_fill} = $_[1] );  }
#line 2407 "fig.pxx"
sub fill_color           {  return ( $#_ == 0 ) ? $_[0]->{fill_color} : ( $_[0]->{fill_color} = $_[1] );  }
#line 2408 "fig.pxx"
sub incrArea_fill        {  return $_[0]->{area_fill} += (defined($_[1]) ? $_[1] : 1); }
#line 2407 "fig.pxx"
sub incrFill_color       {  return $_[0]->{fill_color} += (defined($_[1]) ? $_[1] : 1); }
#line 2405 "fig.pxx"
sub incrLine_style       {  return $_[0]->{line_style} += (defined($_[1]) ? $_[1] : 1); }
#line 2409 "fig.pxx"
sub incrStyle_val        {  return $_[0]->{style_val} += (defined($_[1]) ? $_[1] : 1); }
#line 2406 "fig.pxx"
sub incrThickness        {  return $_[0]->{thickness} += (defined($_[1]) ? $_[1] : 1); }
#line 2405 "fig.pxx"
sub line_style           {  return ( $#_ == 0 ) ? $_[0]->{line_style} : ( $_[0]->{line_style} = $_[1] );  }
#line 2409 "fig.pxx"
sub style_val            {  return ( $#_ == 0 ) ? $_[0]->{style_val} : ( $_[0]->{style_val} = $_[1] );  }
#line 2406 "fig.pxx"
sub thickness            {  return ( $#_ == 0 ) ? $_[0]->{thickness} : ( $_[0]->{thickness} = $_[1] );  }
# End Accessors
###

###
# User Code
#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: XFig::ShapeObject::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::ShapeObject'))) or carp(sprintf('XFig::ShapeObject::initState: NOT $self->isa(XFig::ShapeObject)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
        # thickness => '',
        # fill_color => '',
        # style_val => '',
        # area_fill => '',
        # line_style => '',
    });
}}

#line 2422 "fig.pxx"
sub setFillColor {
    carp('Usage: XFig::ShapeObject::setFillColor(self, color)') unless (@_ == 2);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::ShapeObject'))) or carp(sprintf('XFig::ShapeObject::setFillColor: NOT $self->isa(XFig::ShapeObject)')); 
    my($color) = $_[1]; CORE::ref($color) eq '' or carp(sprintf('XFig::ShapeObject::setFillColor: ref($color) = %s instead of .', CORE::ref($color))); 
#line 2426 "fig.pxx"
    my $ci = &XFig::XFig::colorIndexFor($color);
    if ( defined($ci) ) {
        #print(STDERR "NOTE: Setting fill color to $color ($ci)\n");
    } else {
        print(STDERR "WARN: Failed to find color '$color' - using default\n");
        $ci = $XFig::XFig::Types{color}->{Default};
    }
    $self->fill_color($ci);

    my $areafill = $self->area_fill();
    if ( ( $areafill == -1 ) && ( $XFig::XFig::Defaults{area_fill} == -1 ) ) {
        # The area fill is currently 'no fill', which is also the
        # default area_fill.  Since we are setting the fill color
        # here, it is almost a certainty that we want a different area_fill.
        # We set it to 20 (full saturation of fill color) and leave it to
        # the user to reset if they want something else.
        $self->area_fill(20);
    }

    return $ci;
}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4

=head1 new(Hash $args)
We do not call 'adjustArgs' because our parent already does it.

=cut

=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head2 setFillColor(str $color)
This method takes a symbolic color name and obtains the
corresponding FIG color index.  Returns the color index
on success, returns the default color index on failure.

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         