package XFig::Text;
use strict;

=head1 NAME

XFig::Text -

=head1 SYNOPSIS

use XFig::Text ();

=head1 DESCRIPTION


=head1 NOTES


=head1 EXAMPLES


=head1 IMPLEMENTATION

=cut

use Carp;                      # everyone gets Carp!
use IO::File;                  # everyone gets IO::File!
use IO::File;
use XFig::BasicObject;
use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
@ISA = qw(XFig::BasicObject);
$VERSION = '0.01';

####################### Package Global Variables ###################
use vars qw(%FontSizes);

%FontSizes = (
                         5 => { width =>  75, height => 120, vertical => 130, }, 
                         6 => { width =>  75, height => 120, vertical => 130, }, 
                         7 => { width =>  75, height => 120, vertical => 150, },
                         8 => { width =>  75, height => 135, vertical => 140, }, 
                         9 => { width =>  90, height => 135, vertical => 165, },
                        10 => { width =>  90, height => 135, vertical => 140, }, 
                        11 => { width =>  90, height => 135, vertical => 165, },
                        12 => { width => 105, height => 180, vertical => 210, },
                        13 => { width => 135, height => 195, vertical => 240, },
                        14 => { width => 135, height => 195, vertical => 240, }, 
                        15 => { width => 150, height => 210, vertical => 255, }, 
                        16 => { width => 150, height => 210, vertical => 255, }, 
                        17 => { width => 150, height => 210, vertical => 255, },
                        18 => { width => 165, height => 240, vertical => 285, },
                        19 => { width => 165, height => 240, vertical => 285, },
                        20 => { width => 165, height => 240, vertical => 250, }, 
                        21 => { width => 225, height => 300, vertical => 360, },
                        22 => { width => 225, height => 300, vertical => 360, },
                        23 => { width => 225, height => 300, vertical => 360, },
                        24 => { width => 225, height => 300, vertical => 360, },
                        30 => { width => 300, height => 360, vertical => 480, },
                        40 => { width => 300, height => 360, vertical => 480, },
                         );


############################# Raw Code  ############################
				      
				      
########################## Class Methods ###########################

#line 2214 "fig.pxx"
sub new {
    carp('Usage: XFig::Text::new(class, args)') unless (@_ == 2);
    my($class) = $_[0]; CORE::ref($class) eq '' or carp(sprintf('XFig::Text::new: ref($class) = %s instead of .', CORE::ref($class))); 
    my($args) = $_[1]; ((!$args) || (CORE::ref($args) eq 'HASH')) or carp(sprintf('XFig::Text::new: ref($args) = %s instead of HASH.', ref($args))); 
#line 2215 "fig.pxx"
    $args->{object_code} = 4;
    $args->{sub_type} ||= 0;  # left-justified

    # We have to call 'adjustArgs' even though our parent does it
    # because we need the args before we call the parent.
    &XFig::Object::adjustArgs($args);

    ###
    # The 'height' and 'length' args are not easily computed by users,
    # so this constructor automatically computes theses values given
    # a text string and font size.
    #
    # The 'string' arg must be terminated with '\001'.  To make things
    # easier for users, the 'text' arg is recognized and replaced
    # with 'string' (properly terminated).

    ###
    # Obtain the font size
    $args->{font_size} ||= $XFig::XFig::Defaults{font_size};
    my $size = $args->{font_size};

    ###
    # Obtain the non-terminated $text string.
    my $string = $args->{string};
    my $text   = delete $args->{text};
    if ( $text ) {
        # 'text' is not a legal field (but 'string' is).
        # 'text' is the user-provided string, and 'string'
        # is that string with a \001 suffix.
        if ( $string ) {
            die("ERROR: Do not specify both 'string' and 'text' fields - ambiguous\n");
        }
        $args->{string} = "$text\\001";
    } elsif ( $string !~ m/\\001$/ ) {
        die("ERROR: Using 'text' arg instead of 'string' arg (string must be terminated with 001)");
    } else {
        ($text = $string) =~ s/\\001$//;
    }

    ###
    # Compute height and length
    my ($w,$h,$v) = &XFig::Text::textInfo($size, $text);
    $args->{height} = $h;
    $args->{length} = $w;

    my $self = $class->SUPER::new($args);
    $self->initFields($args, qw(font font_size angle font_flags height length x y string));
    return $self;
}

########################## Instance Methods ########################

###
# Accessors
#line 2195 "fig.pxx"
sub angle                {  return ( $#_ == 0 ) ? $_[0]->{angle} : ( $_[0]->{angle} = $_[1] );  }
#line 2193 "fig.pxx"
sub font                 {  return ( $#_ == 0 ) ? $_[0]->{font} : ( $_[0]->{font} = $_[1] );  }
#line 2196 "fig.pxx"
sub font_flags           {  return ( $#_ == 0 ) ? $_[0]->{font_flags} : ( $_[0]->{font_flags} = $_[1] );  }
#line 2194 "fig.pxx"
sub font_size            {  return ( $#_ == 0 ) ? $_[0]->{font_size} : ( $_[0]->{font_size} = $_[1] );  }
#line 2197 "fig.pxx"
sub height               {  return ( $#_ == 0 ) ? $_[0]->{height} : ( $_[0]->{height} = $_[1] );  }
#line 2195 "fig.pxx"
sub incrAngle            {  return $_[0]->{angle} += (defined($_[1]) ? $_[1] : 1); }
#line 2196 "fig.pxx"
sub incrFont_flags       {  return $_[0]->{font_flags} += (defined($_[1]) ? $_[1] : 1); }
#line 2193 "fig.pxx"
sub incrFont             {  return $_[0]->{font} += (defined($_[1]) ? $_[1] : 1); }
#line 2194 "fig.pxx"
sub incrFont_size        {  return $_[0]->{font_size} += (defined($_[1]) ? $_[1] : 1); }
#line 2197 "fig.pxx"
sub incrHeight           {  return $_[0]->{height} += (defined($_[1]) ? $_[1] : 1); }
#line 2198 "fig.pxx"
sub incrLength           {  return $_[0]->{length} += (defined($_[1]) ? $_[1] : 1); }
#line 2199 "fig.pxx"
sub incrX                {  return $_[0]->{x} += (defined($_[1]) ? $_[1] : 1); }
#line 2200 "fig.pxx"
sub incrY                {  return $_[0]->{y} += (defined($_[1]) ? $_[1] : 1); }
#line 2198 "fig.pxx"
sub length               {  return ( $#_ == 0 ) ? $_[0]->{length} : ( $_[0]->{length} = $_[1] );  }
#line 2210 "fig.pxx"
sub string               {  return ( $#_ == 0 ) ? $_[0]->{string} : ( $_[0]->{string} = $_[1] );  }
#line 2199 "fig.pxx"
sub x                    {  return ( $#_ == 0 ) ? $_[0]->{x} : ( $_[0]->{x} = $_[1] );  }
#line 2200 "fig.pxx"
sub y                    {  return ( $#_ == 0 ) ? $_[0]->{y} : ( $_[0]->{y} = $_[1] );  }
# End Accessors
###

###
# User Code
#line 2306 "fig.pxx"
sub bbox {
    carp('Usage: XFig::Text::bbox(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::bbox: NOT $self->isa(XFig::Text)')); 
#line 2308 "fig.pxx"
    my ($height,$length) = ($self->height(), $self->length());
    my ($x,$y) = ($self->x(), $self->y());
    my $st = $self->sub_type();
    my @result;
    if ( $st == 0 ) {
        # <$x,$y> represents bottom left corner
        push(@result, ($x,$y-$height, $x+$length, $y));
    } elsif ( $st == 1 ) {
        # <$x,$y> represents bottom center
        my $hlen = int($length/2);
        push(@result, ($x-$hlen,$y-$height, $x+$hlen, $y));
    } elsif ( $st == 2 ) {
        # <$x,$y> represents bottom right corner
        push(@result, ($x-$length,$y-$height, $x, $y));
    } else {
        die("ERROR: Invalid XFig::Text subtype $st\n");
    }
    return @result;
}

#line 2294 "fig.pxx"
sub clone {
    carp('Usage: XFig::Text::clone(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::clone: NOT $self->isa(XFig::Text)')); 
#line 2295 "fig.pxx"
    my $result = { %$self };
    bless $result, $self->myclass();
    return $result;
}

#line 2267 "fig.pxx"
sub getText {
    carp('Usage: XFig::Text::getText(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::getText: NOT $self->isa(XFig::Text)')); 
#line 2268 "fig.pxx"
    my $str = $self->string();
    $str =~ s/\\001$//;
    return $str;
}

#line 1 "GENERATED CODE"
sub initState {
    carp('Usage: XFig::Text::initState(self)') unless (@_ == 1);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::initState: NOT $self->isa(XFig::Text)')); 
#line 3 "GENERATED CODE"
{
    $_[0]->SUPER::initState();
    $_[0]->extend({
        # font_size => '',
        # string => '',
        # x => '',
        # angle => '',
        # height => '',
        # font_flags => '',
        # font => '',
        # y => '',
        # length => '',
    });
}}

#line 2300 "fig.pxx"
sub rawTranslate {
    carp('Usage: XFig::Text::rawTranslate(self, dx, dy)') unless (@_ == 3);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::rawTranslate: NOT $self->isa(XFig::Text)')); 
    my($dx) = $_[1]; CORE::ref($dx) eq '' or carp(sprintf('XFig::Text::rawTranslate: ref($dx) = %s instead of .', CORE::ref($dx))); $dx =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Text::rawTranslate: Variable $dx = %s does not match type int.', $dx)); 
    my($dy) = $_[2]; CORE::ref($dy) eq '' or carp(sprintf('XFig::Text::rawTranslate: ref($dy) = %s instead of .', CORE::ref($dy))); $dy =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Text::rawTranslate: Variable $dy = %s does not match type int.', $dy)); 
#line 2302 "fig.pxx"
    $self->x( $self->x() + $dx );
    $self->y( $self->y() + $dy );
}

#line 2273 "fig.pxx"
sub setText {
    carp('Usage: XFig::Text::setText(self, str)') unless (@_ == 2);
    my($self) = $_[0]; ((!$self) || ($self->isa('XFig::Text'))) or carp(sprintf('XFig::Text::setText: NOT $self->isa(XFig::Text)')); 
    my($str) = $_[1]; CORE::ref($str) eq '' or carp(sprintf('XFig::Text::setText: ref($str) = %s instead of .', CORE::ref($str))); 
#line 2274 "fig.pxx"
    $self->string("$str\\001");
}

#line 2289 "fig.pxx"
sub textHeight {
    carp('Usage: XFig::Text::textHeight(font_size)') unless (@_ == 1);
    my($font_size) = $_[0]; CORE::ref($font_size) eq '' or carp(sprintf('XFig::Text::textHeight: ref($font_size) = %s instead of .', CORE::ref($font_size))); $font_size =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Text::textHeight: Variable $font_size = %s does not match type int.', $font_size)); 
#line 2290 "fig.pxx"
    my $height = $FontSizes{$font_size}->{height};
    return $height;
}

#line 2277 "fig.pxx"
sub textInfo {
    carp('Usage: XFig::Text::textInfo(font_size, text)') unless ( (@_ >= 1) && (@_ <= 2) );
    my($font_size) = $_[0]; CORE::ref($font_size) eq '' or carp(sprintf('XFig::Text::textInfo: ref($font_size) = %s instead of .', CORE::ref($font_size))); $font_size =~ m/^[+-]?\d+$/ or carp(sprintf('XFig::Text::textInfo: Variable $font_size = %s does not match type int.', $font_size)); 
    my($text) = $_[1]; $text = '' unless defined($text); CORE::ref($text) eq '' or carp(sprintf('XFig::Text::textInfo: ref($text) = %s instead of .', CORE::ref($text))); 
#line 2283 "fig.pxx"
    my $width    = $FontSizes{$font_size}->{width};
    my $height   = $FontSizes{$font_size}->{height};
    my $vertical = $FontSizes{$font_size}->{vertical};
    return ($width * (CORE::length($text) || 1), $height, $vertical);
}
# User Code
###

###
# Post text

# Post text
###

########################## END OF MODULE ###########################
1;

__DATA__


__END__

=head1 PUBLIC CLASS INTERFACE

=over 4



=head1 PUBLIC INSTANCE INTERFACE

=over 4

=head2 bbox()
Return the bounding box for this object

=cut

=head2 initState()
Automated method for initializing Perl class state based on field definitions.

=cut

=head2 rawTranslate(int $dx, int $dy)
Need only adjust the 'x' and 'y' fields

=cut

=head2 textInfo(int $font_size, str $text = '')
##
Given a font_size and optional text string, returns the
width of the text (or width of one character if no text
given, the height of the text, and the distance from one
line to the next, when stacked vertically

=cut

=head1 AUTHOR

Wade Holst

=head1 SEE ALSO

perl(1).

=cut         